// this program implements a simple hidden Markov Model to infer 
//the presence of copy number polymorphisms, their starting and ending point 
//and the carrier status of each individual

//It models different individuals as realizations of the same Markov chain
//And infers transition probabilities jointly from all individuals

//Version 4+ allows for multiple signals for each probe location, thus allowing for replicate signals
//Version 6+ transforms signals internally into a probability distribution for the states


//Sebastian Zoellner
//5/06

//11/9 Fixed bug regarding sizeof/malloc in the file readin
//11/10 included the option of allowing for overlap (-O option)
//11/13 Fixed a bug in the -t option, cleaned up
//11/14 Changed finddiff_o to change the way the number of CNvs is handeled
//11/16 Implemented struct for CNVs
//2/10 Implemented missing data
//2/14 Redid simulating data; number of carriers is now deterministic
//2/16 Summary statistic calculation for simulation studies
//2/22 Cleaned up memory handling
//3/16 Modelled signal as a mixture of two normals option T4
//3/16 Implemented a second cleaning step requiring signal heterogeneity in finddiff_o

//3/24 Version .6

//4/2 Changed internal storing of mu, sig; all memeory is allocated dynamically now
//4/7 Observation: with the -f flag the likelihood does not have to decrease between iterations
//4/9 Added the -C option to remove singleton CNVs
//4/13 Fixed bug in symmetric option
//4/26 Improved user-friendliness
//5/23 The program now always checks for deletions overlapping with insertions

//8/10 Version .8 Added Viterbi Algorithm for CNV calling
//11/10 Missing data signal can be set in the input file as well.
//11/15 Cleaning Code, allowing for simulation
//12/21 Added method for uniform transition probability
//2/22  Improved output for Viterbi-calls
//3/5	Cleaned up output
//3/18 Fixed problem in output
//3/31 Error when names of lines became too long. Normalizes if mean !=0
//4/9  Outlier Removal reactivated & tweaked
//4/21 Changed calculation of mean and variance of signal to quantile method
//4/25 Some minor I/O changes
//12/something developed EM algorithm to calculate emission probabillities prior to running EM
//1/17 Added a simple normalization option
//1/23 Playing with variable Variances
//7/1 Bughunting 

//2010
//2/8 Include "Prior" by penalizing first probe


#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

struct precn{
  float n;
  long ex;
};

struct cnv{
  int type;
  int start;
  int length;
  float freq;
  float *cs;//carrier status for each individual
  int ov;
  struct cnv *nc;
};

float *** make3arr(int d1,int d2,int d3);
float ** make2arr(int d1,int d2);
struct precn ** make2precarr(int d1,int d2);
double ** make2doubarr(int d1,int d2);

void rem2arr(float **arr,int d1);
void rem2doubarr(double **arr,int d1);
void rem3arr(float *** arr,int d1,int d2);
void rem2precarr(struct precn **arr,int d1);
struct cnv* remcnv(struct cnv * in);

void output(int nos, float * sig, float *mu, int ss, char * name,
	      char * outfile,struct cnv *ecnv);
void vit_out(int nos, int ss, int noce,char * infile,char* outfile, struct cnv *ecnv);

float gasdev(void);
void empr(float *signal,float mu, float sig,int,struct precn *);
float norm(float val);

void read_data(float ***signal,float *** probd, int *noce,int *ss,char * infile,int nosig, int);
void pnmult(struct precn *pn,float mult);
void precsum(struct precn *su1,struct precn su2);
int rescale(struct precn *in,int noe);
void reprec(struct precn *pn);

void assessd(float ***signal, int noce,int ss,float *dmu,float *dsig,int );
void fitmchain(float *** trp,int nos,int noce,int nof);

struct cnv * em(float ***signal,float ***probd, int noce, int nos, int ss, int steps, int noid, 
		int *enoid, float dmu,float dsig, float offset,float minf, int nosig,
		struct cnv *tcnv, struct cnv ** vit, int );

		
struct cnv * simulate_data(float ***signal,int noce, int nos, int ss,
		int ipflag,float * mu, float *sig,int *noid,char *infile,int nosig, int ,float);

void sigcheck(int ss,float ***signal, float *** probd,int nosig,struct cnv *ecnv,char * outfile,int mode);
int trans(int sta);
void memerror(char * fname);

float R2p=2.506628;//root of 2Pi
int Scale=10;
float Bar=1e-10;//sets the number at which digits are moved from .n to the .ex
int Type=4;//flag for the model of signal distribution
int Sy=0;
//float FF=0.0; <- this seems to be unnecessary
int M2=8; 
int M1=0;
int Mode=4;
int Fineflag=0;
int Olflag=0;
float OLP=1e-5;
struct precn Tm;
float Md=99.99;
float Mixprop=0.01;
float Mixfac=10.0;
int B=0;
int Vitflag=0;//Flag for inferring carrier status with the Viterbi algorithm
int Fastflag=0;
int CleanStep=0;
int Normflag=0;
char Simsum[15];
int * Pos=NULL;
char ** Names=NULL;
int Utbflag=0; //Flag for estimating one transition probability for all data, only works with Viterbi calling
int Q_flag=0; //Flag for using quantile methods to estimate mu/sigma
int Int_em_flag=0; //Flag for using an EM to turn intensities into probabillities
float *State_prior;// Weight on different states
float Version =0.813;

/*************************************************************************************************************/
/******************                         Begin of Main                      ***************************** */
/*************************************************************************************************************/
int main(int argc, char * argv[])
{
  void icl(int noa, char *argum[],int *ss, int *nos, int *noce,char * outfile,
	   char * infile,int * sim,int*, float *,float *,int *,int*,float *, float *);
  void check_limits(int noce,int ss, int nos, int nosig);
  void init_out(int ss, int noce, int nos, int nosig, float,float,float,int,int, char *,float,float);
  void calc_probd(float *** signal,float *** probd, float *mu,
		  float * sig, int noce, int nos, int ss, int nosig);
  void est_ms(float *mu, float *sig, struct cnv *ecnv,float ***signal,int ss, int nosig, int nos);

  void repara_em(float *** signal, float *** probd, int noce, int ss, int nosig, int nos,int steps,float min_p,float mins,float);
  void mem_alloc(float ****signal, float ****probd, float **mu,float **sig, int ss,int noce, int nosig,int nos,int tnos,int);

  float *** signal;
  float *** probd;
  int noce=0;
  int nos=3,tnos;//Number of states
  int ss=0;
  int nosig=1; //number of signals for each probe
  char infile[20]="hmmin.txt";
  char outfile[20]="hmmout.txt";
  
  int steps=20;
 
  struct cnv *ecnv;
  struct cnv *vit=NULL;
  struct cnv *tcnv=NULL;

  float *sig,*mu;
  
  int enoid;// Number if detected cnvs 
  float dmu,dsig;
  float offs=0.0; //preset signal strength
  float minf=0.01;
  float min_p=0.05;

  int i,j;
  int olp=0;//removal of outliers that hve prob <10^-olp for all states

  //parameters for simulation
  int simflag=0; //default is real data
  int noid=10; //simulated number of cnvs
  int simsteps =1,slength=6;
  float ses=1.5;//signal strength of CNV
  float maxses=3;
  int maxlength=9;
  int test=1;
  float prior=1.0;

  icl(argc,argv,&ss,&nos,&noce,outfile,infile,&simflag,&steps, &offs,&minf,&nosig,&olp,&min_p,&prior);  
  check_limits(noce,ss,nos,nosig);
  
  if(M2>=1)	
    tnos=(M2+1)*(nos-1)+M1+1;
  else
    tnos=nos;
  
  //memory handling
  mem_alloc(&signal,&probd,&mu,&sig,ss,noce,nosig,nos,tnos,simflag);
  
  
  if (simflag==0 ||test==1||noid==0)// no simulation
    {
      simsteps=1;
      maxlength=slength;
      maxses=ses;
    }


  // for(ses;ses<=maxses;ses+=0.5) 
  //for(j=slength;j<=maxlength;j+=3)//Loops are relevant only for simulation	   
      {
	j=10;

	if(simflag>0)
	  {
	    sprintf(Simsum,"sisu%d%.1f.txt",j,ses);
	    printf("Generating %s\n",Simsum);
	  }
	for(i=0;i<simsteps;++i)   
	  {  
	    //Initial output
	    init_out(ss, noce, nos, nosig,offs,minf,offs, simflag,i,outfile,min_p,prior);
      		
	    if(simflag>0)
	      {	  //Simulating data
		tcnv=simulate_data(signal,noce,nos,ss,simflag,mu,sig,&noid,infile,nosig,j,ses);
		output(nos,sig,mu,ss,"Truth","truthfile",tcnv);
	  
		if(Type==5)
		  {
		    Type=2;		    
		    calc_probd(signal,probd,mu,sig,noce,tnos,ss,nosig);
		    Type=5;
		  }
		printf("Data simulation finished\n");
	      }
	    else 	  //Reading in data
	      {
		printf("starting to read in\n");
		read_data(signal,probd,&noce,&ss,infile,nosig,nos);
		noid=0;
		printf("Data read in\n");
	      }  
  
	    
      
	    //Calculate distributions 
	    if(Type!=5)
	      assessd(signal,noce,ss,&mu[0],&sig[0],nosig);

	    //mu[0]=0.0;mu[1]=1.5;mu[2]=-1.5;sig[0]=1;sig[1]=1;sig[2]=1;
	    if(Int_em_flag==1)
	      {
		repara_em(signal,probd,noce,ss,nosig,nos,steps,min_p,prior,offs);
		Type=5;
	      }
	    for(i=0;i<tnos;++i)
	      State_prior[i]=1.0;
	    for(i=0;i<nos-1;++i)
	      State_prior[M1+(M2+1)*i+1]=prior;
	       
	    ecnv=em(signal,probd,noce,tnos,ss,steps,noid,&enoid,mu[0],sig[0],offs,minf,nosig,tcnv,&vit,olp);

	    //Output
	    if(Type!=5 ||Int_em_flag==1)
	      est_ms(mu,sig,ecnv,signal,ss,nosig,nos); 	

	    if (Utbflag==0)
	      output(nos,sig, mu,ss,infile,outfile,ecnv);
	    if(Vitflag==1)
	      vit_out(nos, ss, noce,infile,outfile,vit);
	    //printf("here3\n");
	    if(Utbflag==0)
	      if(Type!=5)
		sigcheck(ss,signal,probd,nosig,ecnv,outfile,0);
	      else
		sigcheck(ss,signal,probd,nosig,ecnv,outfile,1);
	    tcnv=remcnv(tcnv);
	    ecnv=remcnv(ecnv);
	    vit=remcnv(vit);
	  }
      }
  
  //cleanup
  printf("Cleaning up\n");
  if (Names!=NULL)
    {
      for(i=0;i<ss;++i)
	free(Names[i]);
      free(Names);
    }
  if(Pos!=NULL){
    free(Pos);}
  if(Type!=5 ||simflag>0 )
    rem3arr(signal,ss,noce);
  free(sig);free(mu);
  if( Fastflag==1 ||Type==5)
    rem3arr(probd,ss,noce);
    
  printf("program terminated\n");
  return(0);
}
/*************************************************************************************************************/
/******************                         End of Main                      ***************************** */
/*************************************************************************************************************/

/*****************************************************************************************/
/*****************          DATA SIMULATION            ***********************************/
/******************************************************************************************/

double RandDbl(void)
{
  return rand()/((double)RAND_MAX +1.0);
} 
/******************************************************************************************/
struct cnv * simulate_data(float ***signal,int noce, int nos, int ss,
		   int ipflag,float * mu, float *sig,int *noid,char *infile,int nosig,int length,float es)
{
  float draw(float mu,float stdev);
  void an_known(float ***signal,int noce, int nos, int ss,struct cnv *stt,
		float * mu, float *sig, int nosig);
  void simsamout(float ***signal, int ss, int nosig, char* simout, int noce);

  struct cnv* init_sim(int noce,int nos,int ipflag,float *mu,float *sig,int *noid,char * infile, 
		       int ss, int ,float,float,float);

  int fr;
  float sum;
  int i,k,j;
  struct cnv *stt,*tcnv,*lcnv;
  float mdp=0.0;
  int st,ov;
  float freq=0.1;
  FILE * sf;
char outf[20];


  stt=init_sim(noce,nos,ipflag,mu,sig,noid,infile,ss,length,freq,mdp,es);

  sf=fopen(Simsum,"a");
  fprintf(sf,"Number_of_CNVs\t%d\tlength\t%d\tfrequency\t%.4f\tsignal\t%.4f\t",*noid,length,freq,(mu[1]-mu[0])/sig[0]);
  fprintf(sf,"miss_data\t%.4f\t",mdp);
  fclose (sf);
  
  i=0;

  //draw non-overlapping starting points for CNVs
  for(tcnv=stt;tcnv!=NULL;tcnv=tcnv->nc)
    {
      ++i;
      ov=1;
      while (ov==1)
	{
	  ov=0;
	  st=(int)(RandDbl()*(noce-tcnv->length-2))+1; 
	  for(lcnv=stt;lcnv!=tcnv;lcnv=lcnv->nc)//check overlap
	    if(lcnv->start<st+tcnv->length &&lcnv->start+lcnv->length>st)//overlap
	      ov=1;
	}
      tcnv->start=st;

      printf("Begin of CNV %d (%d): Position %d; length %d; frequency %.2f\n",i,tcnv->type,tcnv->start,tcnv->length,tcnv->freq);	
    }

  //assign carrier status for each individual !!!These are problematic with overlapping CNVs
  
  for(tcnv=stt;tcnv!=NULL;tcnv=tcnv->nc)
    {
      //A		Deterministic 
      for(j=0;j<ss;++j)
	tcnv->cs[j]=0.0;
      fr=(int)ss*tcnv->freq;
      for(j=0;j<fr;++j)
	{
	  k=(int)(RandDbl()*ss);
	  if(tcnv->cs[k]==0.0)
	    tcnv->cs[k]=1.0;
	
	  else
	    --j;
	}
      //B Random
      /*
	for(j=0;j<ss;++j)
	{
	if(RandDbl()<tcnv->freq)
	tcnv->cs[k]=1.0;
	}
      */
    }
  //assign baseline signal to all probes
  for(i=0;i<ss;++i)
    for(k=0;k<noce;++k)
      for(j=0;j<nosig;++j) 
	if(RandDbl()<mdp)
	  signal[i][k][j]=Md;
	else
	  signal[i][k][j]=draw(mu[0],sig[0]);

  //assign non-baseline signal to all probes spanning CNVs
  for(tcnv=stt;tcnv!=NULL;tcnv=tcnv->nc)
    {
      for(i=0;i<ss;++i)
	if(tcnv->cs[i]==1.0)
	  for(j=0;j<tcnv->length;++j)
	    for(k=0;k<nosig;++k)
	      if(RandDbl()<mdp)
		signal[i][tcnv->start+j][k]=Md;
	      else
		signal[i][tcnv->start+j][k]=draw(mu[tcnv->type],sig[tcnv->type]);	
					
    }

  //an_known(signal,noce, nos, ss,stt,mu, sig, nosig);
  sprintf(outf,"simfile");
simsamout(signal,ss,nosig,outf,noce);
  return(stt);
}
/******************************************************************************/
void an_known(float ***signal,int noce, int nos, int ss,struct cnv *stt,
		   float * mu, float *sig, int nosig)
{
  void fw_calc(float ***signal,float ***,int,struct precn **fal,float *mu, float *sig,
	       float ***trp,int samp,float *indist, int noce, int nos,struct precn *li);

  float *** trp;
  float *** empty; //doesn't do anything
  float *indist;
  float *var,*av;
  int *count;
  struct precn **fal;
  struct precn li;
  struct precn tli;
  struct precn t2;
  int i,j,k;
  struct cnv *tcnv;

  //printf("test1\n");

  var=(float *)malloc(nos*sizeof(float));
  av=(float *)malloc(nos*sizeof(float));
  indist=(float *)malloc(nos*sizeof(float));
  count=(int *)malloc(nos*sizeof(int));

  if(var==NULL || av==NULL || indist==NULL ||count==NULL)
    memerror("an_known");
 
  trp=make3arr(noce,nos,nos);
  fal=make2precarr(noce,nos);

  tli.n=1.0;tli.ex=0;t2.n=1.0;t2.ex=0;

  for(i=0;i<nos;++i)
    {count[i]=0;av[i]=0.0;var[i]=0.0;}

  //Generate transition matrix

  for(j=0;j<noce-1;++j)// set true transition probabilities without CNVs   
    for(i=0;i<nos;++i)
      for(k=0;k<nos;++k)
	if(i==k)
	  trp[j][i][k]=1.0;
	else
	  trp[j][i][k]=0.0;
      
  indist[0]=1.0;
  indist[1]=0.00;

  if(nos==3)
    indist[2]=0.00; 

  for(tcnv=stt;tcnv!=NULL;tcnv=tcnv->nc)
    {
      if(tcnv->start>0)
	{
	  trp[tcnv->start-1][0][tcnv->type]=tcnv->freq;
	  trp[tcnv->start-1][0][0]=1-tcnv->freq;
	}
      else
	{
	  indist[tcnv->type]=tcnv->freq;
	  indist[0]=1-tcnv->type;
	}
      trp[tcnv->start+tcnv->length-1][tcnv->type][0]=1.0;
      trp[tcnv->start+tcnv->length-1][tcnv->type][tcnv->type]=0.0;
    }

  for(tcnv=stt;tcnv!=NULL;tcnv=tcnv->nc) 
    for(i=0;i<tcnv->length;++i)
      for(j=0;j<ss;++j)
	for(k=0;k<nosig;++k)
	  if(signal[j][tcnv->start+i][k]!=Md&&tcnv->cs[j]==1.0)
	    {
	      av[tcnv->type]+=signal[j][tcnv->start+i][k];
	      count[tcnv->type]++;
	      var[tcnv->type]+=pow(signal[j][tcnv->start+i][k],2);
	    }

  for(i=0;i<ss;++i)
    { 
      for(j=0;j<noce;++j)
	for(k=0;k<nosig;++k)
	  if(signal[i][j][k]!=Md)
	    {
	      av[0]+=signal[i][j][k];
	      count[0]++;
	      var[0]+=pow(signal[i][j][k],2);
	    }

      fw_calc(signal,empty,nosig,fal,mu, sig,trp,i,indist, noce, nos,&li);
      pnmult(&tli,li.n);
      tli.ex+=li.ex;
    }

  for(i=1;i<nos;++i)
    { 
      av[0]-=av[i];
      count[0]-=count[i];
      var[0]-=var[i];
    }
  for(i=0;i<nos;++i)
    {
      av[i]=av[i]/count[i];
      var[i]=sqrt(var[i]/count[i]-pow(av[i],2));
      if(count[i]>0)
	printf("State %d Count %d  \tAverage signal %.3f  \tVariance  %.3f\n",i,count[i],av[i],var[i]);
      else
	printf("State %d Count %d \n",i,count[i]);
    }
  printf("Prob of chain  %.3e %d\n",tli.n,tli.ex);
  rem3arr(trp,noce,nos);trp=NULL;
  rem2precarr(fal,noce);fal=NULL;
  free(var);free(count);free(av);free(indist);
  return;
}

/*********************************************************************/
struct cnv* init_sim(int noce,int nos,int ipflag,float *mu,float *sig,
		     int *noid,char * infile, int ss, int length, float freq,float mdp, float es)
 {
   struct cnv * read_sim(int *noid,float *mu,float *sig,char * infile, int ss);
   struct cnv * newcnv(int ss);

   int seed=time(NULL); 
   int i;
   struct cnv *ret=NULL,*next;

   //seed=1;

   printf("\n**Simulating data** ");
   if(Type==1)
     printf("binary\n");
   else
     printf("quantitative\n"); 
   
   printf("Seed: \t\t%d\n",seed);
   printf("Number of CNVs:\t%d\n",*noid);
   printf("Length:\t%d\n",length);
   printf("Missing data: \t%.2f\n",mdp);
printf("Signal Model %d\n",Type); 
   printf("Distribution of signals\n");
  

   srand(seed);

   if(ipflag==1)
     {
       if(Type==1)
	 {
	   mu[0]=0.99;
	   mu[1]=0.5;
	 }
       else
	 {
	   mu[0]=0;
	   mu[1]=es;
	   sig[0]=1;
	   sig[1]=1;
	   if(nos==3)
	     {
	       mu[2]=-mu[1];
	       sig[2]=sig[1];
	     }
	 }
      
       for(i=0;i<*noid;++i)  
	 { 
	   if(i==0)
	     {
	       ret=newcnv(ss);
	       next=ret;
	     }
	   else{
	     next->nc=newcnv(ss);
	     next=next->nc;
	   }
	   next->type=1+(int)(RandDbl()*(nos-1));
	   next->length=length;
	   next->freq=freq;
	 }
     }
   else
     if(Type==2 ||Type ==4)
       ret=read_sim(noid,mu,sig,infile,ss);
     else
       {
	 printf(" Only type 2 and type 4 input files supported\n");
	 exit(8);
       }

 for(i=0;i<nos;++i)
    printf("State %d mu %.3f  sig %.3f\n",i,mu[i],sig[i]);

   return (ret);
 }
/*****************************************/
float draw (float mu, float stdev)
{
  float drawd(float prob);
  float drawn(float mu,float stdev);
  float ret;

  switch (Type){
  case 1:
    ret=drawd(mu);
    break;
  case 2:
    ret=drawn(mu,stdev);
    break;
  case 3:
    ret=drawn(mu,stdev);
    break;
  case 4:
    if(RandDbl()<Mixprop)
      ret=drawn(mu,stdev*Mixfac/(1+Mixprop*(Mixfac-1)));
    else
      ret=drawn(mu,stdev/(1+Mixprop*(Mixfac-1)));
    break;
  case 5:
    ret=drawn(mu,stdev);
    break;
  }

  return(ret);
}
/*****************************************/
float drawd(float prob)
{
  float ret;

  if(RandDbl()<prob)
    ret=0.0;
  else
    ret=1.0;
  return(ret);
}
/*****************************************/
float drawn(float mu,float stdev)
{
  float stn=gasdev();
  return(stn*stdev+mu);
}
/*****************************************/
float gasdev(void)
{
       
  static int iset=0;
  static float gset;
  float fac,rsq,v1,v2;

  if  (iset == 0) {
    do {
      v1=2.0*RandDbl()-1.0;
      v2=2.0*RandDbl()-1.0;
      rsq=v1*v1+v2*v2;
    } while (rsq >= 1.0 || rsq == 0.0);
    fac=sqrt(-2.0*log(rsq)/rsq);
    gset=v1*fac;
    iset=1;
    return v2*fac;
  } else {
    iset=0;
    return gset;
  }
}
/**********************************************************************************************/
/**********************HMM-functions***********************************************************/
/*********************************************************************************************/
void fw_calc(float ***signal,float ***probd, int nosig,struct precn **fal,float *mu, float *sig,
	     float ***trp,int samp,float *indist, int noce, int nos,struct precn *li)
{
  int i,j,k;
  float t2;
  double sum;
  struct precn emlik,test;
  int rv;
  //testtrp(nos,noce,trp);
  test.n=1.0;test.ex=0.0;
  //printf("\nindividual %d nos=%d\n",samp,nos);

  for(k=0;k<nos;++k){
    //printf("k=%d\n",k);
    if( Fastflag==1 ||Type==5)
      {
	fal[0][k].n=probd[samp][0][trans(k)];
	fal[0][k].ex=0;
	//printf("k=%d trans= %d %f\n",k,trans(k),probd[samp][0][trans(k)]);
      }
    else
      empr(signal[samp][0],mu[trans(k)],sig[trans(k)],nosig,&fal[0][k]); 

    fal[0][k].n*=State_prior[k];
	
	

    if(indist[k]>0.0 )
      {
	pnmult(&test,fal[0][k].n);
	test.ex+=fal[0][k].ex;
      }
   
    pnmult(&(fal[0][k]),indist[k]);
   
  }
 
  rv=rescale(fal[0],nos);
	
  if(rv==1)
    {
      printf("Error 1 in fw_calc\n");
      exit(8);
    }
  for(i=1;i<noce;++i)
    {
      //printf("probe %d\n",i);
      t2=0.0;
      //printf("probe %d signal %.1f \n",i,signal[samp][i][0]);
      for(j=0;j<nos;++j)
	{ 
	  sum=0.0;
	  if( Fastflag==1 ||Type==5)
	    {
	      emlik.n=probd[samp][i][trans(j)];
	      emlik.ex=0;
	    }
	  else
	    empr(signal[samp][i],mu[trans(j)],sig[trans(j)],nosig,&emlik);
	  emlik.n*=State_prior[j];
	  	
	  for(k=0;k<nos;++k)
	    {
	      sum+=(fal[i-1][k].n*trp[i-1][k][j]);
	      if(fal[i-1][k].n>0.0)
		fal[i][j].ex=fal[i-1][k].ex;
	    }
	  //printf("%d %.1e %.1e %e %e %e %.1e %d %f\n",j,fal[i-1][0].n,fal[i-1][1].n,trp[i-1][0][j],trp[i-1][1][j],trp[i-1][2][j],emlik.n,emlik.ex,signal[samp][i][0]);			
 
	  fal[i][j].n=emlik.n;					//alpha(location i, element j)=sum*emlik
	  fal[i][j].ex+=emlik.ex;
	  //printf("t2\n");
	  pnmult(&(fal[i][j]),sum);
	  //printf("t3\n");
//printf("OP:i %d j %d int:%.3e %d alpha:%.3e %d\n",i,j,emlik.n,emlik.ex,fal[i][j].n,fal[i][j].ex);
	  t2+=fal[i][j].n;
	  if(fal[i][j].n>0.0)
	    {
	      pnmult(&test,emlik.n);
	      test.ex+=emlik.ex;
	      //printf("%d %d \t%.3e %d %.3e %d %.3e %d\n",i,j,fal[i][j].n,fal[i][j].ex,emlik.n,emlik.ex,test.n,test.ex);
	    }
	  
	  if(fal[i][j].n>=-1e50  && fal[i][j].n<=1e50)
	    ;
	  else	   
	    {
	      printf("\nError 2 in fw_calc (%d) state %d : %.3e %d\t%.3e %d (%.3e,%.3e)\n",
		     i,j,fal[i][j].n,fal[i][j].ex,emlik.n,emlik.ex,mu[trans(j)],sig[trans(j)]);
	      for(k=0;k<nos;++k)
		printf("prev state %d %.3e %.3f\n",k,fal[i-1][k].n,trp[i-1][k][j]);
	      exit(8);
	    }
	  
	  
	  //printf("OP2:\t%.3e %d %.3e %d %.3e %d\n",fal[i][j].n,fal[i][j].ex,emlik.n,emlik.ex,test.n,test.ex);
	   
	}
      //printf("brs\n");
      rv=rescale(fal[i],nos);
      //printf("ars\n");

      if(t2==0.0)
	{
	  printf("Error 3 in fw_calc: t2=0.0 (i=%d)\n",i);
	  exit(8);
	}
      		
      if(rv==1)
	{
	  printf("Error 4 in fw_calc (i=%d)\n",i);
	  exit(8);
	}		
      for(j=0;j<nos;++j)
	if(fal[i][j].n<0.0 && fal[i][j].n>1.0)	   
	  {
	    printf("Error 5 in fw_calc: %.3e %d\t%.3e %d\n",fal[i][j].n,fal[i][j].ex,emlik.n,emlik.ex);
	    exit(8);
	  }
    }

  (*li).n=0.0;(*li).ex=0;
  for(i=0;i<nos;++i)
    precsum(li,fal[noce-1][i]);
      
 
  //printf("total likelihood %e %d\n",(*li).n,(*li).ex);
  
  return;
}
/*******************************************************************/
void back_calc(float ***signal,float ***probd,int nosig,struct precn **fal, float * sig, 
	       float * mu,float ***trp,float ***sum_xi,int noce, int nos,
	       float *** gam, int samp, struct precn li)
{
  int i,j,k;
  struct precn tr;
  struct precn *last_be;
  struct precn *be;
  float tsum;
  float *ns;
  struct precn emlik;

  last_be=(struct precn *)malloc(nos*sizeof(struct precn));
  be=(struct precn *)malloc(nos*sizeof(struct precn));
  ns=(float *)malloc(nos*sizeof(float));

  if(last_be ==NULL || be==NULL ||ns==NULL)
    memerror("back_calc");
  
  //printf("\nBackward\n");

  tsum=0.0;
  for(k=0;k<nos;++k)
    {
      last_be[k].n=1.0;
      last_be[k].ex=0;
     
     
      if(fal[noce-1][k].n>0.0)
	gam[samp][noce-1][k]=fal[noce-1][k].n/li.n*pow(10,fal[noce-1][k].ex-li.ex);
      else
	gam[samp][noce-1][k]=0.0;
      
      //printf("%.3e %d  %.3e %d\t",last_be[k].n,last_be[k].ex,gam[samp][noce-1][k].n,
      //gam[samp][noce-1][k].ex);	
    }
  //printf("%.3f %.3e\n",tsum,li.n);

	
  //Testing
  for(j=0;j<nos;++j) 

    if(gam[samp][noce-1][j]>=0.0 && gam[samp][noce-1][j]<=1.01)
      ;
    else
      {	      
	printf(" Error 5: Gamma= %.3e\n",gam[samp][noce-1][j]);
	printf("%d %d\n",noce-1,j);

	for(k=0;k<nos;++k)
	  printf("%.3e %d  %.3e %d\n",fal[noce-1][k].n,fal[noce-1][k].ex,li.n,li.ex);
	     
	exit(8);
      }


  for(i=noce-2;i>=0;--i)
    {
      //printf("i=%d\n",i);
      tsum=0.0;
		
      for(j=0;j<nos;++j)
	{
	  be[j].n=0.0;
	  be[j].ex=0;
			
	  //printf("j=%d\n",j);
	  for(k=0;k<nos;++k)
	    {//printf("j=%d k=%d\n",j,k);
	      if( Fastflag==1 ||Type==5)
		{
		  emlik.n=probd[samp][i+1][trans(k)];
		  emlik.ex=0;
		}
	      else
		empr(signal[samp][i+1],mu[trans(k)],sig[trans(k)],nosig,&emlik);
	      emlik.n*=State_prior[k];

	      tr.n=trp[i][j][k];						//tr(state j at locus i)= transition(i to k)*signal(of k at locus i+1)*beta(state k at loc i+1) 
	      tr.ex=last_be[k].ex+emlik.ex;
	      //printf("here %.2e %d\n",tr.n,tr.ex);
	      pnmult(&tr,1.0);
	      //printf("here2 trp:%.2e %.2e %d\n",trp[i][j][k],tr.n,tr.ex);
	      pnmult(&tr,emlik.n);//printf("here3 em %.2e %.2e %d\n",emlik.n,tr.n,tr.ex);
	      pnmult(&tr,last_be[k].n);
	      //printf("here4 last_be %.2e %.2e %d\n",last_be[k].n,tr.n,tr.ex);
	      if(tr.n<1e50 && tr.n>=0.0)
		;
	      else
		{
		  printf("Error2 in back_calc\n");
		  printf("tr.n =%.3e %.3e %.3e %d %.3e\n",tr.n,trp[i][j][k],emlik.n,emlik.ex,last_be[k].n);
		  exit(8);
		} 
	      //printf("here5 %.2e %d %.2e %d\t",be[j].n,be[j].ex, tr.n,tr.ex);
	      precsum(&(be[j]),tr);
	      //printf("%.2e %d\n",be[j].n,be[j].ex);		
	      if(be[j].n>1e50)		
		printf("Error3 in back_calc : %e\n",be[j].n);
				
	      ns[k]=fal[i][j].n*tr.n/li.n;
	      //printf("\n%d %d tr%.2e em%.2e %d lbet%.2e tr%.2e %d ns:%.2e\n",j,k,trp[i][j][k],emlik.n,emlik.ex,last_be[k].n,tr.n,tr.ex,ns[k]);
	      if(ns[k]>0.0)
		ns[k]*=pow(10,fal[i][j].ex+tr.ex-li.ex);
	      //printf("\n%d %d tr%.2e em%.2e %d lbet%.2e tr%.2e %d ns:%.2e %.2e\n",j,k,trp[i][j][k],emlik.n,emlik.ex,last_be[k].n,tr.n,tr.ex,ns[k],pow(10,fal[i][j].ex+tr.ex-li.ex));
	      if(ns[k]<1.01&&ns[k]>=0.0)
		;
	      else
		{
		  printf("Error in back_calc\n");
		  printf("i=%d,ns[%d] =%.3f fal %.3e %d tr %.3e %d li %.3e %d\n",i,k,
			 ns[k],fal[i][j].n,fal[i][j].ex,tr.n,tr.ex,li.n,li.ex);
		  printf("%d %e\n",fal[i][j].ex+tr.ex-li.ex,pow(10,fal[i][j].ex+tr.ex-li.ex));
		  exit(8);
		} 
	    }
	  //printf("%d %d  %.2e %d\n",i,j,be[j].n,be[j].ex);
	  gam[samp][i][j]=0.0;


	  for(k=0;k<nos;++k)
	    {
	      sum_xi[i][j][k]+=ns[k];
	      gam[samp][i][j]+=ns[k];
	    }

	  tsum+=gam[samp][i][j];	  
 
	}
      //printf("Here$$\n");	
      rescale(be,nos);

      for(j=0;j<nos;++j) 
	{
	  last_be[j].n=be[j].n;
	  last_be[j].ex=be[j].ex;

	  if(be[j].n>1e50)
	    printf("Error4 (%d %d)in back_calc : %e %d\n",i,j,be[j].n,be[j].ex);
	}
      //Testing

      for(j=0;j<nos;++j) 

	if(gam[samp][i][j]>=0.0 && gam[samp][i][j]<=1.01)
	  ;
	else
	  {	      
	    printf(" Error 5: Gamma= %.3e\n",gam[samp][i][j]);
	    printf("%d %d\n",i,j);

	    for(k=0;k<nos;++k)
	      printf("%.3e ",ns[k]); 
	    exit(8);
	  }

		
      /*printf("\n\n%.3e *(%.1e*%.1e+%.1e*%.1e)+\n",fal[noce-2][0].n,trp[noce-2][0][0],
	empr(0,signal[samp][noce-1],mu,sig),trp[noce-2][0][1],
	empr(1,signal[samp][noce-1],mu,sig));
	gsum1=fal[noce-2][0].n*(trp[noce-2][0][0]*
	empr(0,signal[samp][noce-1],mu,sig)+trp[noce-2][0][1]*
	empr(1,signal[samp][noce-1],mu,sig));
	printf("=%e (%e)\n",gsum1,gam[samp][noce-2][0].n*li.n);
	printf("%.3e *(%.1e*%.1e+%.1e*%.1e)\n",fal[noce-2][1].n,trp[noce-2][1][0],
	empr(0,signal[samp][noce-1],mu,sig),trp[noce-2][1][1],
	empr(1,signal[samp][noce-1],mu,sig));
	gsum2=fal[noce-2][1].n*(trp[noce-2][1][0]*
	empr(0,signal[samp][noce-1],mu,sig)+trp[noce-2][1][1]*
	empr(1,signal[samp][noce-1],mu,sig));
	gsum3=(trp[noce-2][1][0]*
	empr(0,signal[samp][noce-1],mu,sig)+trp[noce-2][1][1]*
	empr(1,signal[samp][noce-1],mu,sig));
	printf("=%e(%e)\n",gsum2,gam[samp][noce-2][1].n*li.n);
	printf("=%e(%e)\n",gsum3,last_be[1].ex);*/
      // printf("%.3f %.3e\n",tsum,li.n);
		
      if(tsum<0.99 ||tsum>1.01)
	{
	  printf("Error6 in back_calc\n");
	  printf("test sum =%.3f %.3e\n",tsum,li.n);
	  exit(8);
	}

    }
  //exit(8);
  
  free(last_be);free(be);free(ns);
  return;
}
/***************************************************************************/
void viterbi(float ***signal,float ***probd, int nosig,int* bestest,float *mu, float *sig,
	     float ***trp,int samp,float *indist, int noce, int nos)
{
  int i,j,t;
  int rv;
  struct precn *delta;
  struct precn *det; 
  struct precn nv,emlik;
  int **psi;

  //printf("Starting Viterbi\n");
  delta=(struct precn *)malloc(nos*sizeof(struct precn));
  det=(struct precn *)malloc(nos*sizeof(struct precn));
  psi=malloc(noce*sizeof(int*));
  for (i=0;i<noce;++i)
    psi[i]=malloc(nos*sizeof(int));

  //initializing

  for(i=0;i<nos;++i)
    {
      psi[0][i]=0;
      if( Fastflag==1 ||Type==5)
	{
	  delta[i].n=probd[samp][0][trans(i)];
	  delta[i].ex=0;
	}
      else
	empr(signal[samp][0],mu[trans(i)],sig[trans(i)],nosig,&delta[i]); 
      delta[i].n*=State_prior[i];

      pnmult(&(delta[i]),indist[i]);
      reprec(&delta[i]);
      //printf("%d %f %f %e %d\n",i,mu[trans(i)],sig[trans(i)],delta[i].n,delta[i].ex);
    }
	
  //recursion

  for(t=1;t<noce;++t)
    {//printf("step %d started\n",t);
      //printf(" %.3f %.3f %.3f\n",probd[samp][t][0],probd[samp][t][1],probd[samp][t][2]);
      for(j=0;j<nos;++j)
	{
	  psi[t][j]=0;
	  det[j].n=delta[0].n;
	  det[j].ex=delta[0].ex;
	  pnmult(&det[j],trp[t-1][0][j]);
	  //printf("trp 0 %d : %e  %e %d %e %d\n",j,trp[t-1][0][j], delta[0].n, delta[0].ex,det[j].n,det[j].ex);
	  for(i=1;i<nos;++i)
	    {
	      nv.n=delta[i].n;
	      nv.ex=delta[i].ex;
	      pnmult(&nv,trp[t-1][i][j]);
	      //printf("trp %d %d : %e %e %d %e %d\n",i,j,trp[t-1][i][j], delta[i].n, delta[i].ex,nv.n,nv.ex);
	      if(pncomp(nv,det[j])==1)
		{
		  //printf("Bigger: %e %d\n",nv.n,nv.ex);
		  det[j].n=nv.n;
		  det[j].ex=nv.ex;
		  psi[t][j]=i;
		}
	    }

	  if( Fastflag==1 ||Type==5)	    
	    {      
	      emlik.n=probd[samp][t][trans(j)];
	      emlik.ex=0;	    
	    }	  
	  else	    
	    empr(signal[samp][t],mu[trans(j)],sig[trans(j)],nosig,&emlik);
	  emlik.n*=State_prior[j];

	  //printf("Signal %e %d det %e %d\n",emlik.n,emlik.ex,det[j].n,det[j].ex);
	  pnmult(&det[j],emlik.n);
	  det[j].ex+=emlik.ex;
	}

      for(j=0;j<nos;++j)
	{
	  delta[j].n=det[j].n;	
	  delta[j].ex=det[j].ex;
	  reprec(&delta[j]);
	  //printf("%d:%e %d  (%d)\n",j,delta[j].n,delta[j].ex,psi[t][j]);
	}

	  
      //for(j=0;j<nos;++j)
      //printf("%d %d %d |",t,j,psi[t][j]);
      //printf("step %d finnished\n",t);
    }

  //Termination  
  det[0].n=delta[0].n;	 
  det[0].ex=delta[0].ex;	  
  bestest[noce-1]=0;

  for(i=1;i<nos;++i)		  
    {
	  
      if(pncomp(delta[i],det[0])==1)	  
	{
	  det[0].n=delta[i].n;
	  det[0].ex=delta[i].ex;						  
	  bestest[noce-1]=i;
	}	  
    }

  //Backtracking
  //printf("Backtracking started\n");
  for(t=noce-1;t>0;--t)
    {
      //printf("%d %d %d\n",t,bestest[t],psi[t][bestest[t]]);
      bestest[t-1]=psi[t][bestest[t]];
    }
  //printf("Backtracking finished\n");
  free(delta);
  free(det);
  for (i=0;i<noce;++i)
    free(psi[i]);
  free(psi);

  //printf("Finnishing Viterbi\n");
  return;	
}
/************************************************************************/
void empr(float *signal,float mu, float sig,int nosig,struct precn *emp)
{
  float empr1(float signal,float p0);
  float empr2(float signal,float mu, float sig);
  float empr3(float signal,float nu); //t distribution      
  int i;

  (*emp).n=1.0;
  (*emp).ex=0;
  float sp;
  switch (Type)
    {
    case 1:
      for(i=0;i<nosig;++i)
	if(signal[i]!=Md)
	  pnmult(emp,empr1(signal[i],mu));			  
      break;
    case 2:
      for(i=0;i<nosig;++i)
	if(signal[i]!=Md)
	  pnmult(emp,empr2(signal[i],mu,sig));	
      break;
    case 3:
      for(i=0;i<nosig;++i)
	if(signal[i]!=Md)
	  pnmult(emp,empr3(signal[i]-mu,sig));//centered around 0   
      break;
    case 4:
      for(i=0;i<nosig;++i)
	if(signal[i]!=Md)
	  pnmult(emp,Mixprop*empr2(signal[i],mu,Mixfac*sig)+(1-Mixprop)*empr2(signal[i],mu,sig));
      break;
    case 5:
      printf("For Type 5, this should not be called\n");
      exit(8);
    }
if((*emp).n==0.0)
{
	(*emp).n=1.0;
	(*emp).ex=-100;
}
  return;
}
/***************************************************************************/
float empr1(float signal,float p0)
{  
  //p0 is the probabillity to observe 0
  return (1-signal)*p0+signal*(1-p0);
}
/***************************************************************************/
float empr2(float signal,float mu, float sig)
{
  return (exp(-pow((signal-mu)/sig,2)/2)/(R2p*sig));
}
/***************************************************************************/
float empr3(float signal,float nu) //t distribution     
{
  float beta(float z, float w);
  
  return (pow(1+signal * signal/nu,-(nu+1)/2)/(sqrt(nu)*beta(0.5,nu/2)));
}
/***********************************************************/
void calc_probd(float *** signal,float *** probd, float *mu,
	       float * sig, int noce, int nos, int ss, int nosig)
//Calculates the probability for each state and stores it, to faster run hmm steps that update the transmission probability

{
  int i,j,k,q;
  struct precn *liks=(struct precn *)malloc(nos*sizeof(struct precn));
  struct precn sum;
  float fin;
  int tr_nos=trans(nos);

  //printf("In propd: %d states=%d true states\n",nos,tr_nos);
  if(liks==NULL)
    memerror("calc_probd");

  for(i=0;i<ss;++i)
    for(j=0;j<noce;++j)
      {
	sum.n=0.0;
	sum.ex=0;
	for(k=0;k<tr_nos;++k)
	  {
	    empr(signal[i][j],mu[k],sig[k],nosig,&liks[k]);
	    // printf("%d %.3f %.3f %.3f ",k,signal[i][j][0],mu[k],sig[k]);
	    precsum(&sum,liks[k]);
	    //printf("%d %f %d %f %d\n",k,liks[k].n,liks[k].ex,sum.n,sum.ex);
	  }
	if(sum.n>0.0)
	  for(k=0;k<tr_nos;++k)
	    {
	      probd[i][j][k]=liks[k].n/sum.n*pow(10,liks[k].ex-sum.ex);
	      //printf("%d %d %d %f %f ",i,j,k, liks[k].n,probd[i][j][k]);
	    }
	else
	  for(k=0;k<tr_nos;++k)
	    probd[i][j][k]=1.0/tr_nos;
	//printf("\n");

      }
  
  free(liks);
  return;
}
/***********************************************************************/
/********************Memory handling************************************/
/***********************************************************************/
void mem_alloc(float ****signal, float ****probd, float **mu,float **sig, int ss,int noce, int nosig,int nos,int tnos, int simflag)
{
  if(Type!=5||simflag>0)
    *signal=make3arr(ss,noce,nosig);
  if( Fastflag==1 ||Type==5)//<- This was commented out but should not be?
    *probd=make3arr(ss,noce,nos);
  
  *mu=malloc(nos*sizeof(float));
  *sig=malloc(nos*sizeof(float));

  State_prior=malloc(tnos*sizeof(float));

  if(*mu==NULL || *sig == NULL)
    memerror("mem_alloc");

  printf("Initial memory allocated\n\n");
  return;
}
/******************************************************************************/
struct cnv * newcnv(int ss)
{
  struct cnv * ret=(struct cnv*)malloc(sizeof(struct cnv));
  int i;

  if(ret==NULL)
    memerror("newcnv");
  ret->nc=NULL;
  ret->cs=(float*)malloc(ss*sizeof(float));
  if(ret->cs==NULL)
    memerror("newcnv");
  for(i=0;i<ss;++i)
  ret->cs[i]=0.0;

  return(ret);
}
/************************************************************************/
struct cnv* remcnv(struct cnv * in)
{
  struct cnv * next;
  for(;in!=NULL;in=next)
    {
      next=in->nc;
      free(in->cs);
      free(in);
    }
  return (NULL);
}
/**********************************************************************/
float *** make3arr(int d1,int d2,int d3)
{
  float *** ret;
  int i,j;
  ret=(float ***)malloc(d1*sizeof(float **));
  
  if(ret==NULL)
    memerror("make3arr 1");
  for(i=0;i<d1;++i)
    {
      ret[i]=(float **)malloc(d2*sizeof(float *));
      if(ret[i]==NULL)
	memerror("make3arr 2");

      for(j=0;j<d2;++j)
	{
	  ret[i][j]=(float *)malloc(d3*sizeof(float));
	  if(ret[i][j]==NULL)
	    memerror("make3arr 3");
	}
    }
  return (ret);
}
/*************************************************************************/
void rem3arr(float *** arr,int d1,int d2)
{
  int i,j;

  for(i=0;i<d1;++i)
    {
      for(j=0;j<d2;++j)
	free(arr[i][j]);
      free (arr[i]);
    }
  free(arr);
  return;
}
/***********************************************************************/
float ** make2arr(int d1,int d2)
{
  float ** ret;
  int i;
  ret=(float **)malloc(d1*sizeof(float *));
  if(ret==NULL)
    memerror("make2arr 1");
   
  for(i=0;i<d1;++i)
    {
      ret[i]=(float *)malloc(d2*sizeof(float));
      if(ret[i]==NULL)
	memerror("make2arr 2");     
    }
  return (ret);
}
/*************************************************************************/
void rem2arr(float **arr,int d1)
{
  int i;

  for(i=0;i<d1;++i)
    free (arr[i]);
	
  free(arr);
  return;
}
/***********************************************************************/
double ** make2doubarr(int d1,int d2)
{
  double ** ret;
  int i;
  ret=(double **)malloc(d1*sizeof(double *));
  if(ret==NULL)
    memerror("make2doubarr 1");
   
  for(i=0;i<d1;++i)
    {
      ret[i]=(double *)malloc(d2*sizeof(double));
      if(ret[i]==NULL)
	memerror("make2doubarr 2");     
    }
  return (ret);
}
/*************************************************************************/
void rem2doubarr(double **arr,int d1)
{
  int i;

  for(i=0;i<d1;++i)
    free (arr[i]);
	
  free(arr);
  return;
}
/***********************************************************************/
struct precn ** make2precarr(int d1,int d2)
{
  struct precn **ret;
  int i;
  ret=(struct precn **)malloc(d1*sizeof(struct precn *));
  if(ret==NULL)
    memerror("make2precarr 1"); 
  for(i=0;i<d1;++i)
    {
      ret[i]=(struct precn *)malloc(d2*sizeof(struct precn));
      if(ret[i]==NULL)
	memerror("make2precarr 2");      
    }
  return (ret);
}
/*********************************************************************/
void rem2precarr(struct precn **arr,int d1)
{
  int i;

  for(i=0;i<d1;++i)
    free (arr[i]);
	
  free(arr);
  return;
}
/***********************************************************************/

/************************************************************************/
/*************More HMM***************************************************/
/***********************************************************************/
struct cnv * em(float ***signal,float *** probd,int noce, int nos, int ss, int steps, 
	 int noid, int *enoid, float dmu,
	float dsig, float offset,float minf, int nosig,struct cnv *tcnv, struct cnv ** vit, int outl)
{
  void initialize(float *mu,float *sig,float ***trp, 
		  float *indist,int noce, int nos,float,float, float);
  
  void bout(float ***trp,float * mu,float * sig,
	    char * outfile,float ***gam,
	    int noce,int ss,int nos);

  void run_em(float *mu,float *sig,float ***trp,
	      float *indist,int noce,int nos,int steps,float ***gam,
	      float ***signal, float *** probd, int ss, int, int nosig);

  struct cnv *analyze(float ***trp,float *** gam,int noce,int ss,int nos, int nod, int *enod,
		      float minf,struct cnv *tcnv,float ***signal,float ***probd, int nosig, float * indist, float * mu, float * sig, struct cnv ** vit);

  void hmmout(float *mu,float *sig,float ***trp,
	      float *indist,float ***gam,int noce,int nos, int ss);
void rem_outlier(float *** signal, int ss, int nos, int noce, int nosig,float * mu, float *sig,int outl);
   
  float ***trp=NULL;
  
  float ***gam=NULL;
  struct cnv * ecnv;
  float **test;
  
float *indist=(float *)malloc(nos*sizeof(float));  
  float *sig=(float *)malloc(trans(nos)*sizeof(float));
  float *mu=(float *)malloc(trans(nos)*sizeof(float));

  if(indist == NULL || sig ==NULL || mu==NULL)
    memerror("em");
    

  gam=make3arr(ss,noce,nos); 
  trp=make3arr(noce,nos,nos);
  
  initialize(mu,sig,trp,indist,noce,nos,dmu,dsig,offset);

 if (outl>0)  
	rem_outlier(signal,ss,nos,noce,nosig,mu,sig,outl);

  if(B==0)
    {
      run_em(mu,sig,trp,indist,noce,nos,steps,gam,signal,probd,ss,1,nosig);
   
      //ecnv=analyze(trp,gam,noce,ss,nos,noid,enoid,minf,tcnv);

      if(Fineflag==1)
		  run_em(mu,sig,trp,indist,noce,nos,steps/2,gam,signal,probd,ss,2,nosig);
    }
  else
    run_em(mu,sig,trp,indist,noce,nos,steps,gam,signal,probd,ss,0,nosig);
     
  ecnv=analyze(trp,gam,noce,ss,nos,noid,enoid,minf,tcnv,signal,probd,nosig,indist,mu,sig,vit);
  
  

  free(indist);
  rem3arr(trp,noce,nos);
  rem3arr(gam,ss,noce);
  free(mu);
  free(sig);
  
 
  return(ecnv);
}
/**********************************************************/
void initialize(float *mu,float *sig,float ***trp, 
		float *indist,int noce, int nos,float dmu,float dsig,float offset)
{
  int j,i,k;
  float sigf=1; //Factor by which the variance of the deletion state is modified compared to the variance of the insetion state.
  //float ts;
  printf("Initializing HMM for noce=%d, nos=%d, sigfactor= %.3f\n",noce,nos,sigf);
  for(j=0;j<noce-1;++j)
    
    for(k=0;k<nos;++k)
      {
	for(i=0;i<nos;++i)
	  if(i==k)
	    trp[j][k][i]=(0.9);
	  else
	    trp[j][k][i]=0.1/(nos-1);
	//printf("t %d\n",k);
      }
  
  testtrp(nos,noce,trp);
  fitmchain(trp,nos,noce,2);
  
  indist[0]=(0.95);

  for(i=1;i<nos;++i)
    indist[i]=0.05/(nos-1);
  
  if(Type==4)
    dsig=dsig/sqrt(1-Mixprop*(1-Mixfac));

  mu[0]=dmu;
  mu[1]=dmu+offset*dsig;

  sig[0]=dsig;
  sig[1]=dsig;

  if(Type==3)
    {
      mu[0]=0.0;
      sig[0]=3;//df of t-distr
    }
 
  mu[2]=dmu-offset*dsig;;
  sig[2]=sig[1]*sigf;
   
  return;
}
/*************************************************************************/
void run_em(float *mu,float *sig,float ***trp,float *indist,int noce,int nos,
	    int steps,float ***gam,float ***signal, float *** probd,int ss, 
	    int mode,int nosig)
{
  void est_indist(float *indist,float ***gam,int ss, int nos);
  int est_trp_m2(float ***trp,float ***sum_xi,
		 float ***gam, int noce, int nos, int ss);
  void est_np(float  ***gam,float ***signal,float * mu,
	      float * sig, int noce, int nos, int ss,int nosig);
  void flip(float * indist,float ***trp,float * mu,float * sig,
	    int noce);
  void calc_probd(float *** signal,float *** probd, float *mu,
	       float * sig, int noce, int nos, int ss, int nosig);

  struct precn ** fal;
  float ***sum_xi;
  int i,j,k,m,n;
  struct precn totl,li;
  int testflip=0;
  //printf("in run_em\n");

  sum_xi=make3arr(noce,nos,nos); 
  fal=make2precarr(noce,nos); 

  
  
  if(Fastflag==1 && Type !=5)
    calc_probd(signal, probd, mu,sig,noce,nos,ss,nosig);	 

  //printf("Here:%e %e\n",indist[0],indist[1]);


  for(j=0;j<steps;++j)
    {
      totl.n=1.0;
      totl.ex=0;
      for(k=0;k<noce;++k)
	for(m=0;m<nos;++m)
	  for(n=0;n<nos;++n)
	    sum_xi[k][m][n]=0.0;

      totl.n=1.0;
      totl.ex=0;

      for(i=0;i<ss;++i)
	{//printf("Foreward\n");
	  fw_calc(signal,probd,nosig,fal,mu, sig,trp,i,indist, noce,nos,&li);//printf("Backward\n");	 
	  back_calc(signal,probd,nosig,fal,sig, mu,trp,sum_xi,noce, nos,gam, i,li);
	  pnmult(&totl,li.n);	 
	  totl.ex+=li.ex;	 
	}

      //printf("\n");
      printf("Round %d  lik %e %d ",j,totl.n,totl.ex);
      est_indist(indist,gam,ss,nos);
      
      switch(mode)
	{
	case 0:
	  if(Type!=5)
	    { 
	      est_np(gam,signal,mu,sig,noce,nos,ss,nosig);
	      if(Fastflag==1)
		calc_probd(signal,probd,mu,sig,noce,nos,ss,nosig);
	    }
	  testflip=est_trp_m2(trp,sum_xi,gam,noce,nos,ss);
	  break;
	case 1:
	  testflip=est_trp_m2(trp,sum_xi,gam,noce,nos,ss);
	  break;
	case 2:
	  if(Type!=5)
	    {
	      est_np(gam,signal,mu,sig,noce,nos,ss,nosig);
	     
	      if(Fastflag==1)
		calc_probd(signal,probd,mu,sig,noce,nos,ss,nosig);
	    }
	  break;
	}
      if(Type!=5)
	{
	  for(i=0;i<trans(nos);++i)
	    printf("%.3f %.3f ",mu[i],sig[i]); }
	  printf("\n");
	

      if(testflip==1&&Type==1&&M2==0)
	flip(indist,trp,mu,sig,noce);
     
    } //End of the forewards-backwards
  printf("Forwards-Backwards algorithm finnished.\n\n");
  rem3arr(sum_xi,noce,nos);
  rem2precarr(fal,noce);
  return;
}

/********************************************************/
void flip(float * indist,float ***trp,float * mu,float * sig,int noce)
{
  int i;
  float mem;

  mem=mu[0];
  mu[0]=mu[1];
  mu[1]=mem;

  mem=sig[0];
  sig[0]=sig[1];
  sig[1]=mem;

  mem=indist[0];
  indist[0]=indist[1];
  indist[1]=mem;

  for(i=0;i<noce-1;++i)
    {
      mem=trp[i][0][0];
      trp[i][0][0]=trp[i][1][1];
      trp[i][1][1]=mem;

      mem=trp[i][1][0];
      trp[i][1][0]=trp[i][0][1];
      trp[i][0][1]=mem;
    }
  return;
}
/********************************************************/
void est_indist(float *indist,float *** gam,int ss, int nos)
{
  int i,j;
  double den;
  //Estimates the expected count at each state??

  //printf("estimated indist:\n" ); 
  for(j=0;j<nos;++j)  
    indist[j]=0.0;

  for(i=0;i<ss;++i)
    {
      den=0.0;
      for(j=0;j<nos;++j)
	den+=gam[i][0][j]*ss;

      for(j=0;j<nos;++j)  
	indist[j]+=gam[i][0][j]/den;
    }
  /*
  for(j=0;j<nos;++j) 
     printf("%.3e ",indist[j]);
    
     printf("\n");*/
  return;
}

/****************************************************************************/
void est_np(float ***gam,float ***signal,float * mu,
	    float * sig, int noce, int nos, int ss,int nosig)
{
  void est_np1_m2(float ***gam,float ***signal,float * mu,
		  int noce, int nos, int ss,int nosig);

  void est_np2_m2(float ***gam,float*** signal,float * mu,
		  float * sig, int noce, int nos, int ss,int nosig);

  float * sigrep;//when model 3 is used, sig stands for the deg of freedom
  int i;

  switch (Type)
    {
    case 1:   
      est_np1_m2(gam,signal,mu,noce,nos,ss,nosig);
      break;
    case 2:
      est_np2_m2(gam,signal,mu,sig,noce,nos,ss,nosig);
      break;
    case 3:
      sigrep=malloc(nos*sizeof(float));
      if(sigrep==NULL)
	memerror("est_np");
      est_np2_m2(gam,signal,mu,sigrep,noce,nos,ss,nosig);
      free (sigrep);
      break;
    case 4:
      est_np2_m2(gam,signal,mu,sig,noce,nos,ss,nosig);
      for(i=0;i<trans(nos);++i)
		sig[i]=sig[i]/(1+Mixprop*(Mixfac-1));

      break;
    }
 
  return;
}
/******************************************************************************/
void est_np1_m2(float *** gam,float ***signal,float * mu,
	    int noce, int nos, int ss, int nosig)

//for binary signal
{
  int i,j,k,n;
  double av, sum, *count;
 
  count=malloc(nos*sizeof(double));
  if(count==NULL)
    memerror("est_np1_m2");

 

  for(i=0;i<nos;++i)
    {
      av=0.0;count[i]=0.0;
      for(j=0;j<noce;++j)

	for(k=0;k<ss;++k)
	  for(n=0;n<nosig;++n)
	    if(signal[k][j][n]!=Md)
	      {
		av+=gam[k][j][i]*(1-signal[k][j][n]);
		count[i]+=gam[k][j][i];
	      }
      mu[i]=av/count[i];
      //printf("%d %f\n",i,mu[i]);
    }

  sum=0.0;
  av=0.0;
  for (i=0;i<=M1;++i)
    {
      av+=mu[i]*count[i];
      sum+=count[i];
    }
  for (i=0;i<=M1;++i)
    mu[i]=av/sum;
  sum=0.0;
  av=0.0;
  for(i=M1+1;i<=(M2+1)+M1;++i)
    {
      av+=mu[i]*count[i];
      sum+=count[i];
    }

  for(i=M1+1;i<=(M2+1)+M1;++i)
    mu[i]=av/sum;

  if(nos>(M2+1)+M1)//3 states
    { 
      sum=0.0;
      av=0.0;
      for(i=M2+M1+2;i<=2*(M2+1)+M1;++i)
	{
	  av+=mu[i]*count[i];
	  sum+=count[i];
	}

      for(i=M2+M1+1;i<=2*(M2+1)+M1;++i)
	mu[i]=av/sum;
    }

  free(count);
  return;
}/****************************************************************************/
int est_trp_m2(float ***trp,float ***sum_xi,float *** gam, int noce, int nos, int ss)
//Estimates the transition probabilities. Return value indicates which state is the most common state 
{
	int est_trp_pos(float ***trp,float ***sum_xi, float *** gam, int noce, int nos, int ss);
	int est_trp_av(float ***trp,float ***sum_xi,float *** gam, int noce, int nos, int ss);
	int ret;

	if(Utbflag==0)
	  ret=est_trp_pos(trp,sum_xi, gam, noce, nos, ss);//hmm with time dependent transition prob
	else
	  ret=est_trp_av(trp,sum_xi,gam,noce,nos,ss);//standard old-style hmm

	return (ret);
}
/************************************************************************************/
int est_trp_pos(float ***trp,float ***sum_xi, float *** gam, int noce, int nos, int ss)
{
  int i,j,k,n;
  float sumgam;
  float ts;
  float *ag;
  int rv=0;
  //const int c1=M2+1,c2=2*(M2+1);

  ag=malloc(nos*sizeof(float));
  if(ag==NULL)
    memerror("est_trp_m2");
    

  for(k=0;k<nos;++k)
    ag[k]=0;

  //printf("estimated trp:\n" );
  for(j=0;j<noce-1;++j) { 
    for(k=0;k<nos;++k)
      {
	ts=0.0;   
	sumgam=0.0;

	for(i=0;i<ss;++i)	  
	  sumgam+=gam[i][j][k];
	ag[k]+=sumgam;
	for(n=0;n<nos;++n)
	  {
	    if(sumgam>0.0)		
	      trp[j][k][n]=sum_xi[j][k][n]/sumgam;
	      
	    else
	      trp[j][k][n]=1.0/nos;

	    if(trp[j][k][n]<0.0 ||trp[j][k][n]>1.01)
	      {		
		printf("Error tr %d %d %d = %.4f\n",j,k,n,trp[j][k][n]);
	      }

	    if(trp[j][k][n]<1e-10)
	      trp[j][k][n]=1e-10;
	    
	    //printf("%d%d %.2e\t",k,n,trp[j][k][n]);
	    ts+=trp[j][k][n];
	  }
	if(ts>1.01 ||ts<0.99)
	  {
	    printf("Error in est_trp testsum ts=%.3f\n",ts);
	    exit(8);
	  }	  
      }
    //printf("\n");
  }
  
  fitmchain(trp,nos,noce,2);

  rv=0;
  for(i=1;i<nos;++i)
    if(ag[rv]<ag[i])
      rv=i;
  ts=0;
  for(i=0;i<nos;++i)
    ts+=ag[i];

  for(i=0;i<nos;++i)    
    printf("S%d(%d):%.2f ",i,trans(i),ag[i]/ts);
  
  free(ag);
  return (rv);
}
/*******************************************************************/
int est_trp_av(float ***trp,float ***sum_xi, float *** gam, int noce, int nos, int ss)
{
	int last_mn(int tn)//returns the last modified CNV-number
	{	
		int ret;

		if(tn==0)
			ret=M1;
		else
			ret=M1+tn*(M2+1);

		return (ret);
	}
	int first_mn(int tn)//returns the last modified CNV-number
	{
		int ret;
		if(tn==0)
			ret=0;
		else
			ret=M1+(tn-1)*(M2+1)+1;
		return(ret);
	}
  int i,j,k,n;
  float sumgam;
  float ts;
  double *ag;//average gamma
  double **ax;//average xi
  float **trm;
  int rv=0;
  int tns=trans(nos); //true number of states 
  //const int c1=M2+1,c2=2*(M2+1);

  //printf("entering np\n");
  ag=malloc(tns*sizeof(double));
  ax=make2doubarr(tns,tns);
  trm=make2arr(nos,nos);
  if(ag==NULL ||ax==NULL||trm==NULL)
    memerror("est_trp_m2");
    

  for(k=0;k<tns;++k)
  {
    ag[k]=0.0;

	for(i=0;i<tns;++i)
		ax[i][k]=0.0;
  }

  //printf("estimated trp:\n" );
  for(j=0;j<noce-1;++j) 
 
		 
	  for(k=0;k<tns;++k) 	 
	  {			 
		  for(i=0;i<ss;++i)
			 ag[k]+=gam[i][j][last_mn(k)];			 
		  for(i=0;i<tns;++i)
			  if(i!=k)			  
				  ax[i][k]+=sum_xi[j][last_mn(k)][first_mn(i)];	 
			  else
				  ax[i][k]+=sum_xi[j][last_mn(k)][last_mn(i)];
	  }
//printf("setting up the transition matrix\n");
	  for(i=0;i<nos;++i)
		  for(j=0;j<nos;++j)
			  if(j==i+1)
				  trm[i][j]=1.0;
			  else			  
				  trm[i][j]=0.0;

	  for(i=0;i<tns;++i)
		  for(j=0;j<tns;++j)
		  {
			  if(ag[i]>0)
				  if(i!=j)
					  trm[last_mn(i)][first_mn(j)]=ax[i][j]/ag[i];
				  else
					  trm[last_mn(i)][last_mn(j)]=ax[i][j]/ag[i];
			  else
			  {
				  printf("State %d lost\n",i);
			  exit(8);
			  }
			
			  if(trm[i][j]<0.0 ||trm[i][j]>1.01)
			  {		
			  printf("Error tr %d %d = %.4f\n",i,j,trm[i][j]);
			  }
		  }
		  
//printf("transfering transition matrix\n");

	for(i=0;i<noce-1;++i)
		for(j=0;j<nos;++j)
			for(k=0;k<nos;++k)
				trp[i][j][k]=trm[j][k];
printf("\n");
	for(j=0;j<nos;++j)
			for(k=0;k<nos;++k)
				if(trm[j][k]!=0)
				printf("%d %d %f\n",j,k,trm[j][k]);

//printf("transfered\n");

  for(i=0;i<tns;++i)
    ts+=ag[i];

    
  for(i=0;i<tns;++i) 
  {
    printf("%.2f ",ag[i]/ts); 
	if(ag[rv]<ag[i])
      rv=i;
  }
  free(ag);
  rem2arr(trm,nos);
  rem2doubarr(ax,tns);
  //printf("leaving np\n");
  return (rv);
}

/****************************************************************************/
void est_np2_m2(float ***gam,float ***signal,float * mu,
	       float * sig, int noce, int nos, int ss,int nosig)
{
  //continuous signal
  int i,j,k,n;
  double *av, *count,sd,sum,ac;  
  int nof=2;
  int true_nos=trans(nos);
  int samesigflag=0;//=1 <=> all states have the same sigma
  
  av=malloc(true_nos*sizeof(double));
  count=malloc(true_nos* sizeof(double));
  
  if(av==NULL || count==NULL)
    memerror("est_np2_m2");

  for(i=0;i<true_nos;++i)
    {
      av[i]=0.0;
      count[i]=0.0;
      sig[i]=0.0;
    }
  for(i=0;i<nos;++i)
    {     
      for(j=0;j<noce;++j)
		for(k=0;k<ss;++k)
		for(n=0;n<nosig;++n)
	    if(signal[k][j][n]!=Md)
	      {
		//printf("%f\t%f\n",gam[k][j][i],signal[k][j][n]);	    
		av[trans(i)]+=gam[k][j][i]*signal[k][j][n];
		count[trans(i)]+=gam[k][j][i];
	      }
     
      //printf("%d %d mu= %f %e %e\n",i,trans(i),av[trans(i)]/count[trans(i)],av[trans(i)],count[trans(i)]);
    }
  sum=0.0;
  ac=0.0;
  for(i=0;i<true_nos;++i)
      if(count[i]>0.0)
	  mu[i]=av[i]/count[i];
    

  if (Sy==1)//symmetric
    {
      mu[1]=mu[0]+(count[1]*abs(mu[1]-mu[0])+count[2]*abs(mu[2]-mu[0]))/(count[1]+count[2]);
      mu[2]=2*mu[0]-mu[1];
    }   
  
  for(i=0;i<nos;++i)
    {
     
      for(j=0;j<noce;++j)
	for(k=0;k<ss;++k)
	  for(n=0;n<nosig;++n)
	    if(signal[k][j][n]!=Md)
	      sig[trans(i)]+=gam[k][j][i]*pow((mu[trans(i)]-signal[k][j][n]),2);

      //printf("%d %f %f %f\n",i,sqrt(sig[trans(i)]/count[trans(i)]),count[trans(i)]);
      

    }
  
  sum=0;
  sd=0.0;

  for(i=0;i<true_nos;++i)
    {
    sum+=count[i];
    sd+=sig[i];
	sig[i]=sig[i]/count[i];
    //printf("%f %f %f\n",sum,sd,sig[i]/count[i]);
    }

  if(sum>0.0)
    {
      sd=sqrt(sd/sum);
	  if(samesigflag==1)
	for(i=0;i<true_nos;++i)	
	sig[i]=sd;
      if(sd>=0.0 && sd<1e10)
	;
      else
	{
	  printf("\nError in est_np2_m2; sd= %e\n",sd);
	  exit(8);
	}
      	 
    }
  
  free(av);
  free(count);
  
  return;
}

/************************************************************************/

/******************************************************************************/
/*****Analyzing the HMM output                                         *******/
/****************************************************************************/
struct cnv * analyze(float ***trp,float *** gam,int noce,int ss,int nos, int nod, int *enod,
	     float minf, struct cnv *tcnv,float ***signal,float ***probd, int nosig, float * indist, float * mu, float * sig, struct cnv ** vit)
{
  void compare (struct cnv *ecnv,struct cnv *tcnv, int ss);

  struct cnv * finddiff_o(float ***trp,float *** gam,
			  int noce,int ss,int nos, float minf,int *enod);

  struct cnv *runviterbi(int ss, float ***signal,float ***probd, int nosig,float *mu, float *sig,
	     float ***trp,float *indist, int noce, int nos);

  struct cnv * ecnv=NULL;


  ecnv=finddiff_o(trp,gam,noce,ss,nos,minf,enod);
 
  if(nod>0)//simulated data
    compare (ecnv, tcnv,ss);

if(Vitflag==1)
{
  *vit=runviterbi(ss, signal,probd,nosig,mu, sig,trp,indist, noce, nos);

  if(nod>0)
	  compare(*vit,tcnv,ss);

  // compare (ecnv, vit,ss);
}
else
if(nod>0)//simulated data
    compare (ecnv, tcnv,ss);
 

  return (ecnv);
 }
/************************************************************************************/
void compare (struct cnv * se, struct cnv *st, int ss)
 {
   int i,k;
   int of;
   int dic=0;
   int sterr,lerr;
   int wc;
   float pp;
   struct cnv *ecnv;
   struct cnv *tcnv;
   int nod=0;//number of CNVs
   int tp=0;//true positives
   int fp=0;//false positives
   int fnc=0;
   float fas=0,tas=0,ffas=0;
   int fc,tc,cc;
   float ecpc=0.0,ecpnc=0.0,fcpnc=0.0;
   float pc,nc;
   float poserr=0,pe;
   FILE *sf;
   for(ecnv=se;ecnv!=NULL;ecnv=ecnv->nc)
     ecnv->ov=0;
	
   i=0;
	
   for(tcnv=st;tcnv!=NULL;tcnv=tcnv->nc)
     {
       of=0;
       pp=0.0;
       fc=0;tc=0;pc=0;nc=0;cc=0;
       ++nod;

       for(ecnv=se;ecnv!=NULL;ecnv=ecnv->nc)	   
	 if(ecnv->start<tcnv->start+tcnv->length && ecnv->start+ecnv->length>tcnv->start)
	   if(tcnv->type==ecnv->type)
	     {
	       ++of;
	       for(k=0;k<ss;++k)
		 {
		   cc+=tcnv->cs[k];
		   pp+=fabs(tcnv->cs[k]-ecnv->cs[k]);
			
		   pc+=ecnv->cs[k]*tcnv->cs[k];
		   nc+=ecnv->cs[k]*(1.0-tcnv->cs[k]);

		   if(ecnv->cs[k]>0.1 && tcnv->cs[k]==0.0)
		     ++fc;

		   if(ecnv->cs[k]<0.5 && tcnv->cs[k]==1.0)
		     ++tc;
		 }
	       sterr=abs(ecnv->start-tcnv->start);
	       lerr=abs(tcnv->length-ecnv->length);
	       pe=sterr+abs(tcnv->length+tcnv->start-(ecnv->length+ecnv->start));
			
	       ecnv->ov=1;
		 
	     }

		
       if(of==0)
	 {
	   printf("Indel %d not detected\n",i);
	   ++fnc;
	   //this presently does nothing; to calculate these statistics, we need to go back to the HMM output
	   /*
	     fas+=(float)fc/(ss-cc);printf("%f %d %f\n",fc,ss,cc);
	     ecpnc+=nc/(ss-cc);
	   */
	 }
       else
	 {
	   printf("Indel %d detected by %d dels; average prediction error %.3f\n"
		  ,i,of,pp/(of*ss));
	   printf("Start of last off by %d length off by %d\n",sterr,lerr);
	   dic+=of;
	   ++tp;
	   fas+=(float)fc/(of*ss-cc); 
	   /*if(of*ss-cc==0.0){
	     printf("T: %d %d %d %d dat %d\n",fc,of,ss,cc, of*ss-cc);exit(8);}*/
	   tas+=(float)tc/cc;
	   ecpc+=pc/cc;
	   ecpnc+=nc/(of*ss-cc);
	   //printf("T2: %f %d %d %d dat %d\n",nc,of,ss,cc, of*ss-cc);
	   poserr+=pe;
	
	 }
       i++;
     }
   pp=0.0;
   for(ecnv=se;ecnv!=NULL;ecnv=ecnv->nc)
     if(ecnv->ov==0)//False positives
       {  
	 ++fp;
	 lerr+=ecnv->length;
	 for(k=0;k<ss;++k)
	   {
	     pp+=ecnv->cs[k];
	     //printf("%.1f ",ecnv->cs[k]);
	     if(ecnv->cs[k]>0.1)
	       ++ffas;

		
	     if(pp>=0.0)
	       ;
	     else {
	       printf("Error pp=%f %d %d\n",pp,ecnv->start, ecnv->length);exit(8);
	     }
	   }
	 //printf("\n");
       }
   fcpnc+=pp/(fp*ss);
   //printf("%f %f %d %d\n",fcpnc,pp,fp,ss); 
   if(fp>0)
     printf(" %d estimated dels have no overlap, total length %d, exist in %f\n",fp,lerr,pp/(fp*ss));

   sf=fopen(Simsum,"a");
   if(tp>0)
     {
       fprintf(sf,"True_positives\t%d\tAv_cp_of_carriers\t%.4f\tav_cp_of_noncarriers\t%.4f\tcar_below_0.5\t%.4f\tnoncar_ab_0.1\t%.4f\t",tp,ecpc/tp,ecpnc/tp,tas/tp,fas/tp);
       fprintf(sf,"Av_position_error\t%.4f\t",poserr/tp);
     }
   else{
     fprintf(sf,"True_positives\t0\tAv_cp_of_carriers\t0\tav_cp_of_noncarriers\t0\tcar_below_0.5\t0\tnoncar_ab_0.1\t0\t");
     fprintf(sf,"Av_position_error\t0\t");
   }
   if(fp>0)	   
     fprintf(sf,"False_positives\t%d\tav_cp\t%.4f\tnoncar_ab_0.1\t%.4f\tav_length\t%.4f\t",fp,fcpnc,ffas/fp,(float)lerr/fp);
   else 
     fprintf(sf,"False_positives\t0\tav_cp\t0\tnoncar_ab_0.1\t0\tav_length\t0\t");

   fprintf(sf,"\n");

   fclose(sf);
   printf ("%d true positives  %d false positives\n",tp,fp);
   return;
 }
 /*******************************************************************************/
 struct cnv *runviterbi(int ss, float ***signal,float ***probd, int nosig,float *mu, float *sig,
	     float ***trp,float *indist, int noce, int nos)
 {
   struct cnv *checkex(struct cnv *ret,int st,int len,int t);

   int i,j,t,k;
   int *bestest;
   struct cnv *ecnv=NULL,*last,*ret=NULL;
   int st, len;
   int nof=trans(nos);

   int count=0;
   printf("\n\nViterbi summary\n");
   bestest=malloc(noce*sizeof(int));
	 
   for(i=0;i<ss;++i)
     {//printf("Individual %d\n",i);
       viterbi(signal,probd, nosig,bestest,mu, sig,trp,i,indist, noce, nos);
       //printf("Viterbi done\n");
       for(j=0;j<noce;++j)
	 {
	   //printf("j=%d\n",j);
	   if(bestest[j]>M1)
	     for(t=1;t<nof;++t)
	       if(bestest[j]>M1+(t-1)*(M2+1)&& bestest[j]<M1+1+t*(M2+1))
		 {
		   printf("individual %d\tstart %d",i,j);
		   ++count;
		   st=j;
		   while(bestest[j]>M1+(t-1)*(M2+1)&& bestest[j]<M1+1+t*(M2+1)&&j<noce)
		     ++j;
		   len=j-st;
	
		   printf("\tlength %d\n",len);
		   //check if it exists
		   if(ret!=NULL)
		     {
		       ecnv=checkex(ret,st,len,t);
		       if(ecnv==NULL)
			 {
			   last->nc=newcnv(ss);//make new cnv
			   last=last->nc;
			   last->type=t;
			   last->start=st;
			   last->length=len;
			   for(k=0;k<ss;++k)
			     last->cs[k]=0;
			   last->cs[i]=1;
			 }
		       else
			 ecnv->cs[i]=1;
		     }
		   else
		     {
		       //printf("adding new cnv\n");
		       ret=newcnv(ss);
		       ret->type=t;
		       ret->start=st;
		       ret->length=len;

		       ret->cs[i]=1.0;
		       last=ret;
		       //printf("new cnv added\n");
		     }
		   --j;
		 }
			
	 }
     }
   for(last=ret;last!=NULL;last=last->nc)
     {
       k=0;
       for(i=0;i<ss;++i)
	 k+=last->cs[i];
       last->freq=(float)k/ss;
     }
   printf("Viterbi: %d CNVs found\n",count);
   free(bestest);
   return(ret);
 }
/****************************************************************************/ 
struct cnv *checkex(struct cnv *ret,int st,int len,int t)
{
	while(ret->type != t || ret->start!=st ||ret->length !=len)
	{
ret=ret->nc;
if(ret==NULL)
break;
	}
	return(ret);
}

/********************************************************************************/
struct cnv *finddiff_o(float ***trp,float *** gam, int noce,int ss,int nos, float minf,int *enod)
{
  int i,j,k;
  float ts;
  int nof=trans(nos);//true number of states
  struct cnv *ecnv=NULL;
  struct cnv * st=ecnv;
  struct cnv *rm,*last;
  int rem=0;
  char extest;
  float exthresh=0.2;
	
 
  *enod=0;
 
  printf("\n\nHeuristic CNV calling minimal frequency %f\n",minf);

  if(CleanStep==1 && (2*exthresh/ss)>minf)
  {
	  printf("\nWarning: -C option and minimal frequency of %.4f result in wonky results\n,",minf);
		printf("Actual minimal frequency is at least %.3f\n\n",2*exthresh/ss);
  }
for(i=1;i<nof;++i)
  for(j=0;j<noce-M2-1;++j)      
    //for(i=0;i<nof && j<noce-M2-1;++i)
      if(trp[j][M1][M1+1+(i-1)*(M2+1)]>minf)
	{
	 
	  if(ecnv==NULL)
	    {
	      st=newcnv(ss);
	      ecnv=st;
	    }
	  else
	    {
	      st->nc=newcnv(ss);
	      st=st->nc;
	    }
	    

	  //printf("%d %d\n",j,noce-M2-1);
	  st->start=j+1;
	  st->type=i;
	  st->freq=trp[j][M1][M1+1+(i-1)*(M2+1)];

	  ts=0.0;
	 
	   
	  for(k=M2+1;ts<0.95 && k+j<noce;++k)	    
	    ts+=trp[j+k][M1+(1+M2)*i][0];
	  
	  st->length=k-1;
	  
	  if(Olflag==0)
	    j+=k;
	  ++*enod;
	  printf("CNV %d Type %d start %d frequency %.3f length %d\n",*enod,st->type,st->start,st->freq,st->length);
	
	}    

  printf("\nCarry probabilities:\n"); 
  last=NULL;
  for(st=ecnv;st!=NULL;)
    {
      extest=0;
      for(k=0;k<ss;++k)
	{	
	  //carry probabilty for each individual is the average probability of all probes covered by the CNV
	  st->cs[k]=0.0;

	  for(j=0;j<M2+1;++j)
	    {
	      st->cs[k]+=gam[k][j+st->start][M1+(st->type-1)*(M2+1)+(j+1)];
	      //printf("A %d %.2f ",j,gam[k][j+st->start][M1+(st->type-1)*(M2+1)+(j+1)]);
	    }
	  for(;j<st->length;++j)
	    {
	      st->cs[k]+=gam[k][j+st->start][st->type*(M2+1)+M1];
	      //printf("B %d %.2f ",j,gam[k][j+st->start][st->type*(M2+1)+M1]);
	    }
	  st->cs[k]=st->cs[k]/st->length;
	  //printf("%.1f %d\n",st->cs[k],st->length);
	  if(st->cs[k]>exthresh)
	    ++extest;
	}
      //printf("\n%d\n",extest);
      // if no individual has a carry probability above X%, the CNV is removed
      if (extest<2 && CleanStep==1)
	{
	  rm=st;
	  st=st->nc;

	  free(rm->cs);
	  free(rm);

	  if(last==NULL)
	    ecnv=st;
	  else
	    last->nc=st;
	  ++rem;
	  --*enod;
	}
      else
	{
	  last=st;
	  st=st->nc;
	}
  
    }
  printf("%d CNPs detected %d removed in secondary step\n",*enod, rem);
  
  return(ecnv);
 }

/***************************************************************/
/*******************I/O*****************************************/
/***************************************************************/
/***************************************I/O*****************************************************/
void icl(int noa, char *argum[],int *ss, int *nos, int *noce,char * outfile,
	 char * infile,int * sim,int * steps, float * offset, float *minf,int *nosig,int *otl,float * min_p, float *prior)
{
  void helpout(void);
  float defaultoffset(int minl)
  {
	float ret[11];
	ret[0]=6.5;
	ret[1]=5;
	ret[2]=4.1;
	ret[3]=3.6;
	ret[4]=3.1;
	ret[5]=2.8;
	ret[6]=2.4;
	ret[7]=2.3;
	ret[8]=2.1;
	ret[9]=2.1;
	ret[10]=2;

	if(minl>10) minl=10;

	return (ret[minl]);
  }

  //*sim=0;

	if(noa==1)
	{
	helpout();
	exit(8);
	}

  while((noa>1) && argum[1][0]=='-')
    {
      switch (argum[1][1])
	{
	case 'r':/*Rounds of EM*/
	  *steps=atoi(&argum[1][2]);
	  break;
	case 's':/*Sample Size*/
	  *ss=atoi(&argum[1][2]);
	  break;
	case 'p':/*Number of probes*/
	  *noce=atoi(&argum[1][2]);
	  break;
	case 'n':/*number of states*/
	  *nos=atoi(&argum[1][2]);
	  break;
	case 'T':/*Type of signal:1 binary; 2 normal; 3 t-distr; 4 Mixture of normals; 5 input probs*/
	  Type=atoi(&argum[1][2]);
	  break;
	case 'b'://Mixture proportion for Type 4
		Mixprop=atof(&argum[1][2]);
		break;
	case 'S':/*analyze simulated data 0=take data from file 
		   1= use default parameters
		   2=input parameters*/
	  *sim=atoi(&argum[1][2]);
	  break; 
	case 'o':/*outputfile*/
	  strcpy(outfile,&argum[1][2]);
	  break;
	case 'i':/*inputfile*/
	  strcpy(infile,&argum[1][2]);	  
	  break;
	case 'm':/*minimum length of CNV*/
	  M2=atoi(&argum[1][2])-1;
	  break;
	case 'h'://help
	  helpout();
	  break;
	case 'a'://average factor of the CNV state
	  *offset= atof(&argum[1][2]);
	  break;
	case 'd'://minimum distance between CNPs
	  M1=atoi(&argum[1][2])-1;
	  break;
	case 't'://Threshold for CNV calling
	  *minf=atof(&argum[1][2]);
	  break;
	case 'N'://number of signals for each probe
		*nosig=atoi(&argum[1][2]);
		break;
	case 'F'://Flagg for better estimate of p-values
	  Fineflag=1;
	  break;
	case 'C'://Cleanup: Remove all singletons after analysis
		CleanStep=1;
	break;
	case 'O'://Flag to allow overlap between CMVs
	  Olflag=1;
	  break;
	case 'M'://code for missing data
		Md=atof(&argum[1][2]);
	  break;
	case 'f'://fast analysis, uses more memory
	  Fastflag=1;
	  break;
	case 'B'://analysis that does not preset mean & variance
		B=1;
		break;
	case 'V'://Viterbi algorithm used to infer carrier status
		Vitflag=1;	
		break;
	case 'H'://flag for classical HMM with one transition matrix
		Utbflag=1;
		Vitflag=1;
		break;
	case 'R'://remove outliers with p<10-x
		*otl=atoi(&argum[1][2]);
		break;
	case 'Q'://use quantile method to generate initial estimates
		Q_flag=1;
		break;
	case 'E'://Estimate signal distributions with an EM	  
	  Int_em_flag=1;
	  *min_p=atof(&argum[1][2]);
	  break;
	case 'P'://prior, only useful with E flag
	  *prior=atof(&argum[1][2]);
	  break;
	case 'A'://Normalize by subtracting the average signal
		Normflag=1;
		break;
	default:
	  fprintf(stderr,"\nbad option %s\n",argum[1]);
	  helpout();
	  exit(8);
	  break;
	}
      --noa;
      ++argum;
    }

  if(*offset==0 && B==0 && Int_em_flag==0 && Type!=5)
  {
	  *offset=defaultoffset(M2);
	  printf("Offset (-a option) not set. Using default value %.1f\n", *offset); 
  }
  return;
}
/****************************************************/
void helpout(void)
{

  printf("This program implements a simple hidden Markov Model\n"); 
  printf("to infer the presence of copy number polymorphisms,\n");
  printf("their starting and ending point and the carrier status\n");
  printf("of each individual\n\n");

  printf("It models different individuals as realizations of the \n");
  printf("same Markov chain and infers transition probabilities \n");
  printf("jointly from all individuals.\n\n");

  printf("Options:\n");

  printf("Data Input:\n");
  printf("  s\tSample Size\n");
  printf("  p\tNumber of probes\n");
  printf("  n\tnumber of states\n");
  printf("  o\toutputfile\n");
  printf("  i\tinputfile\n");
  printf("  N\tnumber of signals for each probe\n");
  printf("  M\tCode for missing data\n");

  printf("\nAnalysis:\n");
  printf("  r\tRounds of EM\n");
  printf("  T\tType of signal:1 binary; 2=normal, 3=t-distr,\n\t4=Mixture of Normals, 5 Input of probabilities\n");
  printf("  b\tProportion of signals from large variance normal\n");
  printf("  a\taverage factor of the CNV state\n"); 
  printf("  m\tminimum length of CNV\n");
  printf("  d\tminimum distance between CNVs\n");
  printf("  t\tminimum threshold for CNV calling\n"); 
  printf("  F\tRun second hmm to improve estimate of p-values\n");
  printf("  O\tFlag to allow overlap between CNVs of the same type\n"); 
  printf("  C\tCleanup: Remove all singletons after analysis\n");
  printf("  B\tNo preset factor for the CNV state\n");
  printf("  V\tViterbi algorithm used to infer carrier status\n");
  printf("  H\tflag for classical HMM with one transition matrix\n");
  printf("\nNormalisation:\n");
  printf("  A\tNormalize by subtracting the grand mean\n"); 
  printf("  Rx\tRemove datapoints that are outliers (p<1-^-x)\n");
  printf("  Q\tUse Quantile method for the initial estimation of mu/sigma\n");
  printf("  E\tEstimate signal distributions with an EM\n");  
  printf("  P\tPrior on belonging to a non-baseline state, only useful with E flag\n");


  printf("\nMiscelanious:\n");
  printf("  f\tFast analysis, uses more memory\n");
  printf("  h\thelp; this output\n");
  printf("  S\tanalyze simulated data \n\t0=take data from file (real data)\n\t1= use default parameters\n\t2=input parameters\n");
 
  
  exit(1);
}
/****************************************************/
void check_limits(int noce,int ss, int nos,int nosig)
{
  int er=0;
  
  if(ss==1 && CleanStep==1)
    {
      printf("No CNV can be detected with sample size 1 and the -C option\n");
      printf("The -C option requires a CNV to occur in at least two individuals\n");
      printf("Disabeling -C \n");
      CleanStep=0;
    }
  if(Sy==1&& Type==2 && nos!=3){
    printf("Symmetric means only work for 3 states\nSymmetric means disabeled\n");
    Sy=0;
  }
 
  if(M2<0){
    printf("Minimal CNV length 1 probe\n");
    er=1;
  }

  if(noce<2)
    {
      printf("At least 2 probes required\n");
      er=1;
    }
  if(er==1)
    exit(8);
  
  return;
}
/************************************************************/
void init_out(int ss, int noce, int nos, int nosig, float signalstr,float minf,float signal,int simflag, int simnum, char * outfile, float min_p,float prior)
  {
    FILE *sf;
    time_t now;

    time(&now);
    printf("Program version\t%.3f\n",Version);
    printf("Started on\t%s\n",ctime(&now));
    if (simflag>0)
      {
	printf("starting sim %d\n",simnum+1);
	sf=fopen(Simsum,"a");
	fprintf(sf,"Sample_size \t%d\tNumber_of_probes \t%d\tSignals\t%d\tsignal_strength\t%.4f\t",ss,noce,nosig, signalstr);
	fprintf(sf,"Minimal_length\t%d\tMin_distance\t%d\tCall_frequency\t%.3f\t",M2+1, M1+1,minf);
	fclose (sf);
      }

    printf("Output file %s\n",outfile);
    printf("Sample size \t\t%d\nNumber of probes \t%d\nNumber of states \t%d\nSignals per probe \t%d\n",ss,noce,nos,nosig);
    printf("Minimal length of CNP \t%d\nMin distance btw CNPs \t%d\n",M2+1, M1+1);
    
    if(Int_em_flag==1)
      printf("using EM to estimate carrier probabillities at each marker; minimum probabillity = %.3f, prior =%.3f\n",min_p,prior);
    else
      {    
	printf("Signal intensity of CNVs in standard deviations\t%.2f\n",signal); 
	switch(Type){
	case 1:
	  printf("Binary signal\n");
	  break;
	case 2:
	  printf("Normally distributed signal\n");
	  break;
	case 3:
	  printf("t distributed signal (3df)\n");
	  break;
	case 4:
	  printf("Mixture distribution of normals; proportion of outliers %.4f, factor %.1f\n",Mixprop, Mixfac);
	  break;
	case 5: 
	  printf("Input values are probabilities\n");
	  break;
	default:
	  printf ("Signal type %d not supported\n",Type);
	  break;
	}
	if (prior !=1.0)
	  printf("\tWeight ag CNV:\t%.2e\n",prior);
      }
    printf("\n");
    printf("Minimum calling frequency\t%.3f\n",minf);

    if (Fastflag==1)
      printf("Fast algorithm\n");



    return;
  }
/********************************************************************************************************/
void hmmout(float *mu,float *sig,float *** trp,
	    float *indist,float ***gam,int noce,int nos, int ss)
{
  int i,j,k;
  char outf[15]="hmmoutp.txt";
  FILE * OF;

  OF=fopen (outf,"w");
  fprintf(OF,"%d states\n%d probes\n%d samples\n",nos,noce,ss);
  fprintf(OF,"\nEstimated Emission Model:\n");
  for(i=0;i<nos;++i)
    fprintf(OF,"%d\t%.4f\t%.4f\n",i,mu[i],sig[i]);
  fprintf(OF,"\nEstimated Starting Probabilities:\n");
  for(i=0;i<nos;++i)
    fprintf(OF,"%d\t%.4f\n",i,indist[i]);
  fprintf(OF,"\nEstimated Transition Probabilities:\n");

  for(i=0;i<noce-1;++i)
    {
      fprintf(OF,"%d ",i);
      for(j=0;j<nos;++j){
	fprintf(OF,"|");
	for(k=0;k<nos;++k)
	  if(j!=k)
	    fprintf(OF,"%.2f ",trp[i][j][k]);
      }
      fprintf(OF,"\n");
    }

  fprintf(OF,"\nEstimated State Probabilities:\n");
  for(i=0;i<noce;++i)
    {fprintf(OF,"%d ",i);
      for(j=0;j<ss;++j){
	fprintf(OF,"|");
	for(k=1;k<nos;++k)
	  fprintf(OF,"%.2f ",gam[j][i][k]);
      }
      fprintf(OF,"\n");
    }

  fclose(OF);
  return;
}
/*******************************************************************/
void simsamout(float ***signal, int ss, int nosig, char* simout, int noce)
{
	FILE *OP=fopen(simout, "w");
	int i,j,k;

	fprintf (OP,"# Output of simulated dataset\n");
	fprintf(OP,"P %d %d\n",noce,ss);

	for(i=0;i<ss;++i)

	{
	for(k=0;k<noce;++k)
		for(j=0;j<nosig;++j)
			fprintf(OP,"%.5f ",signal[i][k][j]);

	fprintf(OP,"\n");
	}
fclose(OP);
return;
}
/***********************************************/
void read_data(float ***signal, float *** propd,int* noce, int *ss, char * infile, int nosig,int nos)
{
  float readsignal(char * temp, float ***signal,float ***,int sa,int noce, int nosig, int);
  void readposinfo(char * temp, int noce);
  void readnames(char * temp, int ss,int maxnamelength);

  FILE *seq_file=fopen(infile,"r");
  char *temp=NULL;
  float *tump=NULL;
  char *st_ret=NULL;
  int css=0;
  float avsig=0.0;
  int rnoce,rss;
  int ts;
  int i;
  int maxnamelength=32;

  if(seq_file==NULL)
    {
      printf("Error opening data file %s\n",infile);
      exit(8);
    }
  else
    {
      ts=20*(*noce)*nosig;
    
      //printf("length of string: %d %d %d\n",ts,sizeof(temp),sizeof(tump));
      temp=(char *)malloc(ts);
    
      //printf("length of string: %d %d %d\n",ts,sizeof(temp),sizeof(tump));
    
      if(temp==NULL)
	memerror("read_data");

      // st_ret=fgets(temp,sizeof(temp),seq_file);
     
      st_ret=fgets(temp,ts,seq_file);
      while(st_ret!=NULL)
	{ 
	  //printf("reading:%s\n",temp);
	  switch (temp[0])
	    {
	    case '#':
	      /*comment, ignore*/
	      break;

	    case 'P':
	      sscanf(temp,"P %d\t%d",&rnoce,&rss);
	      if(rnoce!=*noce || rss!=*ss)			
		{
		  printf("file parameters %d %d different from input parameters %d %d \n",
			 rnoce,rss,*noce,*ss);
		  printf("Input line %s\n",temp);
		  exit(8);
		}
	      break;
	    case 'L':		  
	      Pos=malloc(*noce*sizeof(int));
	      readposinfo(temp,*noce);
	      printf("%d Positions read in\n",*noce);
	      break;
	    case 'N':
	      Names=(char **)malloc(*ss*sizeof(char *));
	      for(i=0;i<*ss;++i)
		Names[i]=(char *)malloc(maxnamelength*sizeof(char));
	      readnames(temp,*ss,maxnamelength);
	      printf("%d Names read in\n",*ss);
	      break;
	    case 'M':
	      sscanf(temp,"M %f",&Md);
	      printf("Missing data flagg %f\n",Md);
	      break;

	    default :
	      avsig+=readsignal(temp,signal,propd,css,*noce,nosig,nos);	     
	      css++;  
	      //printf("line %d read\n",css);
	      break;
	    }

	  //st_ret=fgets(temp,sizeof(temp),seq_file);
	  st_ret=fgets(temp,ts,seq_file);
	}
      if(css!=*ss)
		
	{
	  printf("Input error, wrong sample size\n");
	  exit(8);
	}
    }
  avsig=avsig/ *ss;

  //printf("Average signal : %f\n",avsig);
  fclose(seq_file);
  free(temp);
  return;
}
/************************************************************/
void readposinfo(char * temp, int noce)
{
  int k,j,c;
  int sl=strlen(temp);
  int sig;
  
  c=0;
  j=1;
  for(k=1; k<=sl && j==1;)
    {
      j=sscanf(&temp[k],"%d ",&sig); 
      if(j==1)
	{
	  Pos[c]=sig;
	  ++c;
	if (c>noce) 
	  {
	    printf("Input error individual in position information: too many datapoints\nexpected %d\t observed %d\n",noce,c);
	    exit(8);
	  }  
	while(temp[k]<=32 && k<=sl)
	  ++k;      
	while(temp[k]>32 && k<=sl)
	  ++k;   	 
	while(temp[k]<=32 && k<=sl)//produces error in valgrind, bc I may be out of bounds
	  ++k;    
	}
    }
  return;
}/************************************************************/
void readnames(char * temp, int ss, int maxnamelength)
{
  int k,s,c;
  int sl=strlen(temp);
  
  c=0;
  k=1;

  while(temp[k]<=32 && k<=sl)
	  ++k;
  while(k<sl)
  {
	  s=k;
	  while(temp[k]>32 && k<=sl && k-s<maxnamelength-2)
	  {
		  Names[c][k-s]=temp[k];
		  ++k;  
		  
	  }
	  Names[c][k-s]='\0';
	  if (k-s==maxnamelength-2)
			  printf ("Line name truncated to %c\n",Names[c]); 
	  ++c;
	
	  if (c>ss) 
	  {
	    printf("Input error individual in position information: too many datapoints\nexpected %d\t observed %d\n",ss,c);
	    exit(8);
	  }  
	      
	while(temp[k]<=32 && k<=sl)
	  ++k;
  }
    
  return;
}
/***************************************************************/

float readsignal(char * temp, float ***signal,float *** probd,int sa,int noce,int nosig,int nos)
{
  int i,j,n;
  int k=0;
  float sig;
  float av=0.0;
  int sl=strlen(temp);
float cs;//checksum

  i=0;j=1;n=0;cs=0;
  //printf("Individual %d %s\n",sa+1,temp);
  
  while(j==1&&k<=sl)
    {
      j=sscanf(&temp[k],"%f ",&sig); 
      if(j==1)
	{
	  if(Type==5)
	    {
	      probd[sa][i][n]=sig;
		  cs+=sig;
	      ++n;
	      if(n==nos)
		{
			if(cs>1.01||cs<0.99)
			{
				printf("Error in input line %d probe %d %f<>1.0\n",sa,i,cs);
				exit(8);
			}
			cs=0.0;
		  n=0;
		  ++i;
		}
	    }
	  else
	    {
	      signal[sa][i][n]=sig;
	      av+=sig;
	      //if(i%1000==0)	      printf("%d\t%f\n",i,sig);
	      ++n;
	      if(n==nosig)
		{
		  n=0;
		  ++i;
		}
	    }
	  if (i>noce) 
	    {
	      printf("Input error individual %d: too many datapoints\nexpected %d\t observed %d\n",sa+1,noce,i);
	      exit(8);
	    }  
     
     
	  while(temp[k]<=32 && k<=sl)
	    ++k;      
	  while(temp[k]>32&& k<=sl)
	    ++k;   	 
	  while(temp[k]<=32&& k<=sl)//produces error in valgrind
	    ++k;    
	}
    }
  
  if(i!=noce){
    printf("Input error individual %d; wrong number of datapoints\nexpected %d\t observed %d\n",sa+1,noce,i);
    exit(8);
  }
  av=av/noce;
  
  return (av);
}
/**********************************************************************/
struct cnv * read_sim(int *noid,float *mu,float *sig,char * infile, int ss)
{
  FILE *sim_file=fopen(infile,"r");
  char temp[100];
  char *st_ret;
  int css=0;
  int noi=0,nod=0;
  int cnoi=0,cnod=0;
  struct cnv *ret=NULL,*next;
  int type, length; float freq;

  *noid=0;
  if(sim_file==NULL)
    {
      printf("Error opening data file %s\n",infile);
      exit(8);
    }
  else{
 
    st_ret=fgets(temp,sizeof(temp),sim_file);
     
    while(st_ret!=NULL)
      { 
	//printf("reading:%s\n",temp);
	switch (temp[0])
	  {
	  case '#':
	    /*comment, ignore*/
	    break;

	  case 'D':
	    sscanf(temp,"D %d\t%f\t%f",&nod,&mu[1],&sig[1]);
	    printf("nod =%d mu[1]= %f, sig[1]=%f\n",nod,mu[1],sig[1]);
	    *noid+=nod;	    
	    break;
	  case 'I':
	    sscanf(temp,"I %d\t%f\t%f",&noi,&mu[2],&sig[2]);
	    printf("noi =%d mu[2]= %f, sig[2]=%f\n",noi,mu[2],sig[2]);
	    *noid+=noi;	    
	    break;
	  case 'N':
	    sscanf(temp,"N %f\t%f",&mu[0],&sig[0]);
	    printf(" mu[0]= %f, sig[0]=%f\n",mu[0],sig[0]);
	        
	    break;
	  default :
	    sscanf(temp,"%d\t%f\t%d",&type,&freq,&length);
	    if(type==1) ++cnod;
	    if(type==2) ++cnoi;
		if(ret==NULL){
			ret=newcnv(ss);
			next=ret;
		}
		else
		{
			next->nc=newcnv(ss);
			next=next->nc;
		}
		next->type=type;
		next->freq=freq;
		next->length=length;
	    css++;  
	    break;
	  }
	
	st_ret=fgets(temp,sizeof(temp),sim_file);
      }
    if(css!=*noid)
      {
	printf("Input error, wrong number of indels (%d != %d)\n",css,*noid);
	exit(8);
      }
    if(cnoi !=noi){
      printf("Wrong number of insertions (%d !=%d)\n",cnoi,noi);
      exit(8);
    }

    if(cnod !=nod){
      printf("Wrong number of CNVs (%d !=%d)\n",cnod,nod);
      exit(8);
    }
  }
  printf("\nSimulation parameters read in from file %s\n",infile);
  fclose(sim_file);
  return(ret);
}/**************************************************************************/
 void output(int nos, float * sig, float *mu, int ss, char * name,
	      char * outfile,struct cnv *ecnv)
 {
   int j,i;
   FILE *OF;
   time_t now;
   time(&now);

   printf("in output\n");
   OF=fopen(outfile,"w");
   fprintf(OF,"Output file for %s\n",name);
   fprintf(OF,"Generated on %sVersion %.3f\n",ctime(&now),Version);
   fprintf(OF,"Signal:\n");
   for(j=0;j<nos;++j)
     fprintf(OF,"%d %.3f  ",j,mu[j]);
   fprintf(OF,"\n");
   //printf("test1\n");
   if(Type>1)
     for(j=0;j<nos;++j)
       fprintf(OF,"%d %.3f  ",j,sig[j]);
   fprintf(OF,"\n");

   if(Names!=NULL)
     {
       for(j=0;j<ss;++j)
	 fprintf(OF,"%s ",Names[j]);
       fprintf(OF,"\n");
     }
   i=0;
   //printf("test2\n");
   while(ecnv!=NULL)   
     {
       ++i;
       if(Pos!=NULL)
	 fprintf(OF,"CNV %d type %d start %d position %d length %d end position %d frequency %.3f ",i,ecnv->type,ecnv->start,Pos[ecnv->start],ecnv->length,Pos[ecnv->start+ecnv->length-1],ecnv->freq);
       else
	 fprintf(OF,"CNV %d type %d start %d length %d frequency %.3f ",
		 i,ecnv->type,ecnv->start,ecnv->length,ecnv->freq);

       fprintf(OF,"Carrier status: ");
       for(j=0;j<ss;++j)
	 fprintf(OF,"%.2f ",ecnv->cs[j]);
       fprintf(OF,"\n");
	   
       ecnv=ecnv->nc;
     }

   fclose(OF);printf("output generated\n");
   return;
 }
/********************************************************************************/
 
void vit_out(int nos, int ss, int noce,char * infile, char* outfile, struct cnv *ecnv)
{
	
  struct cpc
  {
    int pn;
    struct cpc * np;
  };
  struct cpc *cprobes=NULL, *nc;
  int i,t;
  struct cnv *cc;
  float co=0.25;
  char outvit[25]="vit_";
  time_t now;
printf("in Vit_out\n");
  time(&now);
  strcat(outvit,outfile);

  FILE * OF;
  OF=fopen(outvit,"w");
  fprintf(OF,"Viterbi output file for %s\n",infile);
  fprintf(OF,"Generated on %sVersion %.3f\n",ctime(&now),Version);

  //find all probes covered by a cnv somewhere
  for (i=0;i<noce;++i)
    {
      cc=ecnv;
      t=0;
      while(t==0 &&cc!=NULL)
	{
	  if(i>=cc->start &&i<cc->start+cc->length)
	    t=1;
	  cc=cc->nc;
	}
      if(t==1)
	if(cprobes==NULL)
	  {
	    cprobes=malloc(sizeof(struct cpc));
	    nc=cprobes;
	    nc->pn=i;
	  }
	else
	  {
	    nc->np=malloc(sizeof(struct cpc));
	    nc=nc->np;
	    nc->pn=i;			
	  }
    }
  nc->np=NULL;

  //output
  nc=cprobes;
  if(Names!=NULL)
    printf("\t");
  while(nc!=NULL)
    {
      fprintf(OF,"%d\t",nc->pn);
      nc=nc->np;
    }
  fprintf(OF,"\n");
  if(Pos!=NULL)
    {
      nc=cprobes;
      if(Names!=NULL)
	printf("\t");

      while(nc!=NULL)
	{
	  fprintf(OF,"%d\t",Pos[nc->pn]);
	  nc=nc->np;
	}
      fprintf(OF,"\n");
    }
  for (i=0;i<ss;++i)
    {
      if(Names!=NULL)
	fprintf(OF,"%s\t",Names[i]);
      nc=cprobes;
      while(nc!=NULL)
	{
	  cc=ecnv;
	  t=0;
	  while(t==0 &&cc!=NULL)
	    {
	      if(nc->pn>=cc->start &&nc->pn<cc->start+cc->length&&cc->cs[i]>co)
		t=cc->type;
	      cc=cc->nc;
	    }
	  fprintf(OF,"%d\t",t);
	  nc=nc->np;
	}
      fprintf(OF,"\n");
    }
  fclose(OF);
  if(cprobes!=NULL)
    {
      while(cprobes->np!=NULL)
	{
	  nc=cprobes->np;
	  free(cprobes);
	  cprobes=nc;
	}
      free(cprobes);
    }
  printf("Leaving Vit_out\n");
  return;
}
/********************************************************************************/
/****************************************************************************/
void sigcheck(int ss,float ***signal, float *** probd,int nosig,struct cnv *ecnv,char * outfile,int mode)
{
  int i=0,j,k,n;
  float av;
  float as;
  int sc;
  FILE *OF;
  char fn[50]="support_";
 
  strcat(fn,outfile);
  OF=fopen(fn,"w");

  fprintf(OF,"-------------------------------------------------------\n");
if(mode==0)
  fprintf(OF,"Signal at inferred CNVs for each probe in each individual:\n");
 if(mode==1)
 fprintf(OF,"Probabillity at inferred CNVs for each probe in each individual:\n");
  while(ecnv!=NULL)
    {
      ++i;
      fprintf(OF,"CNV %d type %d\n",i,ecnv->type);
      for(j=0;j<ss;++j)
	{
	  fprintf(OF,"%.2f: ",ecnv->cs[j]);
	  av=0.0;sc=0;
	  for(k=0;k<ecnv->length;++k)
	    {	     
	      if(mode==0)
		{
		  as=0.0;
		for(n=0;n<nosig;++n)		
		  as+=signal[j][k+ecnv->start][n];
		as=as/nosig;
		}
	      if(mode==1)
		as=probd[j][k+ecnv->start][ecnv->type];
	      
	      fprintf(OF,"%.2f ",as);
	      if(as!=Md)
		{
		  av+=as;
		  ++sc;
		}
	    }
	  fprintf(OF,"av:%.2f\n",av/sc);
	} fprintf(OF,"\n");
      ecnv=ecnv->nc;
    }
  fprintf(OF,"-------------------------------------------------------\n");
  fclose(OF);
  return;
}
/**********************************************************************/
void est_ms(float *mu, float *sig, struct cnv *ecnv,float ***signal,int ss, int nosig, int nos)
{
  int i,j,k;
  float *count=malloc(nos*sizeof(float));
  struct cnv *cnv;
  
  for(i=1;i<nos;++i)
    {
      mu[i]=0.0;
      sig[i]=0.0;
    }

  for(cnv=ecnv;cnv!=NULL;cnv=cnv->nc) 
       
    for(j=0;j<ss;++j)
      for(k=0;k<cnv->length;++k)
	for(i=0;i<nosig;++i)
	  {
	    mu[cnv->type]+=cnv->cs[j]*signal[j][k+cnv->start][i];
	    count[cnv->type]+=cnv->cs[j];
	  }
    
  for(i=1;i<nos;++i)
    mu[i]=mu[i]/count[i];
  
  for(cnv=ecnv;cnv!=NULL;cnv=cnv->nc)
    
    for(j=0;j<ss;++j)
      for(k=0;k<cnv->length;++k)
	for(i=0;i<nosig;++i)			
	  sig[cnv->type]+=cnv->cs[j]*pow(signal[j][k+cnv->start][i]-mu[cnv->type],2);
  
  for(i=1;i<nos;++i)
    sig[i]=sig[i]/count[i];
    
    free(count);
  return;
}
/******************************************************************/
void bout(float ***trp,float * mu,float * sig,char * outfile, float ***gam,int noce,int ss,int nos)
{
  int i;
  FILE* OP=fopen(outfile,"w");

 printf("T2:%f %f\n",gam[3][0][0],***(gam+3));

  fprintf(OP,"0->1\t");
  for (i=0;i<nos-1;++i)
    fprintf(OP,"%.2f ",trp[i][0][1]);
  fprintf(OP,"\n");
  fprintf(OP,"1->0\t");
  for (i=0;i<nos-1;++i)
    fprintf(OP,"%.2f ",trp[i][1][0]);
  fprintf(OP,"\n");fprintf(OP,"\n");
  fprintf(OP,"0->2\t");
  for (i=0;i<nos-1;++i)
    fprintf(OP,"%.2f ",trp[i][0][2]);
  fprintf(OP,"\n");
  fprintf(OP,"2->0\t");
  for (i=0;i<nos-1;++i)
    fprintf(OP,"%.2f ",trp[i][2][0]);
  fprintf(OP,"\n");fprintf(OP,"\n");

fclose(OP);
  return;
}
/******************************************************************************/
void assessd(float ***signal, int noce,int ss,float *dmu,float *dsig, int nosig)
{
	//Evaluating the distribution of signals; calculating mean and stdv for state 0
  void quantnorm(float ***signal, int noce,int ss,float *mu,float *sig, int nosig,float avg);
	
  struct dce{
    float val;
    struct dce *p;
    struct dce *n;
  };
  int i,j,n;
  float *sum;
  float ssu;
  int *mdc;
  int tmdc=0;

  

  *dmu=0.0;
  *dsig=0.0;

  sum=malloc(ss*sizeof(float));
  mdc=malloc(ss*sizeof(int));

  if(sum==NULL || mdc==NULL)
    memerror("assessd");

  for(i=0;i<ss;++i)
    {
      sum[i]=0.0;
      mdc[i]=0;
      for(j=0;j<noce;++j)
	for(n=0;n<nosig;++n)
	  if(signal[i][j][n]==Md)
	    mdc[i]++;
	  else
	    sum[i]+=signal[i][j][n];
      *dmu+=sum[i];
      tmdc+=mdc[i];
      sum[i]=sum[i]/(noce*nosig-mdc[i]);
    }
  *dmu=*dmu/(ss*noce*nosig-tmdc);


  for(i=0;i<ss;++i)
    {
      ssu=0.0;
      for(j=0;j<noce;++j)
	for(n=0;n<nosig;++n)
	  if(signal[i][j][n] !=Md)
	    {
	      ssu+=pow(sum[i]-signal[i][j][n],2);
	      *dsig+=pow(*dmu-signal[i][j][n],2);
	    }
      printf("indiv %d mean %f sig %f missing data %d\n",i,sum[i],sqrt(ssu/(noce*nosig-mdc[i])),mdc[i]);
    }
  *dsig=sqrt(*dsig/(ss*noce*nosig-tmdc));

  printf("mean; %f  standard deviation %f total missing data %d\n",*dmu,*dsig, tmdc);
  if(Q_flag==1)
    {
      quantnorm(signal, noce,ss,dmu,dsig, nosig,*dmu);
      printf("Quantile normalization: mean; %f  standard deviation %f total missing data %d\n",*dmu,*dsig, tmdc);
    }
  if(Normflag==1)
    {
      for(i=0;i<ss;++i)
	for(j=0;j<noce;++j)
	  for(n=0;n<nosig;++n)
	    signal[i][j][n]=signal[i][j][n]-*dmu;
      printf("Mean normalized to 0\n");
      *dmu=0.0;
    }
  else
    if(*dmu<-0.05 || *dmu>0.05)
      printf("WARNING: mean different from zero and no normalization\n");
  free(sum);free(mdc);
  return;
}
/**********************************************************************/
void quantnorm(float ***signal, int noce,int ss,float *mu,float *stdv, int nosig,float avg)
  {
    struct dce{
      float val;
      struct dce *p;
      struct dce *n;
    };
    int i,j,n;

    struct dce *start;
    struct dce *low,*sig, *high;
    int fw=0;
    int lp,mp,hp;
    float lv,hv;

    for(n=0;n<nosig;++n)
      for(i=0;i<ss;++i)
	for(j=0;j<noce;++j)

	  if(signal[i][j][n] !=Md)
	    {
	      if(fw==0)
		{
		  start=malloc(sizeof(struct dce));
		  start->val=signal[0][0][0];
		  start->n=NULL;
		  start->p=NULL;
		  low=start;
		  high=start;
		  fw=1;
		}
	      else
		{
		  //printf("%d %d %d\n",i,j,n);
		  sig=malloc(sizeof(struct dce));
		  sig->val=signal[i][j][n];

		  if(sig->val<avg)	
		    if(low->val>=sig->val)
		      {
			sig->p=NULL;
			sig->n=low;
			low->p=sig;
			low=sig;
		      }
		    else
		      {
			start=low;
			while(start->val<sig->val&&start->n!=NULL)
			  start=start->n;
			if(start->n==NULL)
			  {
			    sig->n=NULL;
			    sig->p=start;
			    start->n=sig;
			  }
			else
			  {
			    sig->n=start;
			    sig->p=start->p;
			    sig->n->p=sig;
			    sig->p->n=sig;
			  }
		      }
		  else
		    if(high->val<=sig->val)
		      {
			sig->n=NULL;
			sig->p=high;
			high->n=sig;
			high=sig;
		      }
		    else
		      {
			start=high;
			while(start->val>sig->val&&start->p!=NULL)
			  start=start->p;
			if(start->p==NULL)
			  {
			    sig->p=NULL;
			    sig->n=start;
			    start->p=sig;
			  }
			else
			  {
			    sig->p=start;
			    sig->n=start->n;
			    sig->n->p=sig;
			    sig->p->n=sig;
			  }
		      }
		}
	    }
    printf("done sorting\n");
    /*
      for(start=low;start!=NULL;start=start->n)
      printf("%f\n",start->val);
    */
    fw=ss*noce*nosig;
    lp=0.1*fw;mp=0.5*fw;hp=0.9*fw;
    start=low;
    for(i=0;i<lp;++i)
      start=start->n;

    lv=start->val;

    for(;i<mp;++i)
      start=start->n;

    *mu=start->val;

    for(;i<hp;++i)
      start=start->n;

    hv=start->val;
    printf ("%d points %d %d %d low %f high %f median %f\n",fw,lp,mp,hp,lv,hv,*mu);
    *stdv=(hv-lv)/2.564;
    printf("cleaning up\n");
    start=low;
    while(low!=NULL)
      {
	low=start->n;
	free(start);
	start=low;
      }
    return;
  }
/***************************************************************************/
void reprec(struct precn *pn)
{
	if((*pn).n>0.0)
    {
      while((*pn).n<Bar)
	{
	  (*pn).n=(*pn).n*pow(10,Scale);
	  (*pn).ex-=Scale;
	}
      while((*pn).n>1/Bar)
	{
	  (*pn).n=(*pn).n/pow(10,Scale);
	  (*pn).ex+=Scale;
	}
    }
  else
    (*pn).ex=0;
}
/**************************************************************************/
void pnmult(struct precn *pn,float mult)
{
  int ad=0;
  
  //printf("%.2e %d %.2e =\n",(*pn).n,(*pn).ex,mult);
  if(mult>0.0 && (*pn).n >0.0)
    {
      while(mult<Bar)
	{
	  mult=mult*pow(10,Scale);
	  ad-=Scale;
	}
      while(mult>1/Bar)
	{
	  mult=mult/pow(10,Scale);
	  ad+=Scale;
	}

      (*pn).n*=mult;
      //printf("A%.2e %d (%.2e )\n",(*pn).n,(*pn).ex,mult);
      while((*pn).n<Bar)
	{
	  (*pn).n=(*pn).n*pow(10,Scale);
	  ad-=Scale;
	}
      //printf("B%.2e %d (%.2e )\n",(*pn).n,(*pn).ex,mult);
      while((*pn).n>1/Bar)
	{
	  (*pn).n=(*pn).n/pow(10,Scale);
	  ad+=Scale;
	}
      //printf("C%.2e %d\n",(*pn).n,(*pn).ex);
      (*pn).ex+=ad;
    }
  else
    {
      if(mult<0.0 || (*pn).n <0.0)
	{
	  printf("pnmult not defined for negative numbers : %f %f \n",mult,(*pn).n);
	  exit(8);
	}

      (*pn).n=0.0;
      (*pn).ex=0;
    }
  return;
}
/******************************************************************************/
int pncomp(struct precn val1,struct precn val2)
{
	//compares two numbers of type precn, returns 1 if val1 is bigger, 0 otherwise

	int ret=0;

	reprec(&val1);
	reprec(&val2);

	if((val1.ex>val2.ex && val1.n>0&&val2.n>0) || (val1.n>val2.n &&val1.n*val2.n<=0)||(val1.ex<val2.ex && val1.n<0 && val2.n<0))
	{ret=1;}
	else
		if(val1.n>val2.n &&val1.ex==val2.ex)
			{ret=1;}

	return(ret);
}

/******************************************************************************/
void precsum(struct precn *su1,struct precn su2)
{ 
	//I'm not sure this works for negative numbers

  if(su2.n>0.0 && (*su1).n>0.0)
    {
      while((*su1).ex>su2.ex)
	{
	  su2.n=su2.n/pow(10,Scale);
	  su2.ex+=Scale;
	}

      while((*su1).ex<su2.ex)
	{
	  (*su1).n=(*su1).n/pow(10,Scale);
	  (*su1).ex+=Scale;
	}
    }
  else   
    if(su2.n>0.0)
      (*su1).ex=su2.ex;
    
  (*su1).n+=su2.n;


  //printf("%.2e %d\n",(*su1).n,(*su1).ex);
  reprec(su1);
  
  //printf("%.2e %d\n",(*su1).n,(*su1).ex);
  return;
}
/*****************************************************************************/
int rescale(struct precn *in,int noe)
{
  int i;
  struct precn sum;
  int rv=0;

  sum.n=0.0;sum.ex=0;

  for(i=0;i<noe;++i)
    precsum(&sum,in[i]);

  if(sum.n>=-1e20 &&sum.n<1e20)
    ;
  else
    rv=1;
  //printf("%.2e %d\n",sum.n,sum.ex);
  for(i=0;i<noe;++i){
    if(in[i].n>0)
      {
	while(in[i].ex>sum.ex)
	  {
		  printf("You are here?\n");exit(8);
	    in[i].n=in[i].n*pow(10,Scale);
	    in[i].ex-=Scale;

	  }
	while(in[i].ex<sum.ex)
	  {
	    in[i].n=in[i].n/pow(10,Scale);
	    in[i].ex+=Scale;
	  }
      }
    else
      in[i].ex=0;
    //printf("after rescale %.2e %d ",in[i].n,in[i].ex);
  }
  //printf("\n");
  return (rv);
}
/*****************************************************************/
float gammln(float xx)
//Returns the value ln[Γ(xx)] for xx > 0.
{
  //Internal arithmetic will be done in double precision, a nicety that you 
  //can omit if five-figure accuracy is good enough.
double x,y,tmp,ser;
static double cof[6]={76.18009172947146,-86.50532032941677,
24.01409824083091,-1.231739572450155,
0.1208650973866179e-2,-0.5395239384953e-5};
int j;
y=x=xx;
tmp=x+5.5;
tmp -= (x+0.5)*log(tmp);
ser=1.000000000190015;
for (j=0;j<=5;j++) ser += cof[j]/++y;
return -tmp+log(2.5066282746310005*ser/x);
}
/**************************************************************************/
float beta(float z, float w)
//Returns the value of the beta function B(z,w).
{
float gammln(float xx);
return exp(gammln(z)+gammln(w)-gammln(z+w));
}
/********************************************************/
void fitmchain(float *** trp,int nos,int noce,int nof)//this is only writen for three states and should be generalized
{
  int i,j,k,n;
  float ts;
  int mem;

  //printf("%d %d\n",M1,M2);


  for(j=0;j<noce-1;++j)  
    for(k=0;k<nos;++k)
      {
	
	  if(k<M1)
	    {
	      for(i=0;i<nos;++i)		
			  trp[j][k][i]=0.0;
	      trp[j][k][k+1]=1.0;
	    }
	  if(k==M1)
	    for(i=0;i<nos;++i)
	      if(k!=i && i!=M1+1 && i!= M1+M2+2)		
			  trp[j][k][i]=0.0;


	if(k>M1)
	  {
	    mem=0;
	    for(n=0;n<nof;++n)	  	  	    		    
	      if(k==M1+(1+M2)*(n+1))
		{
		  //printf("%d!!!\n",k);
		  for(i=0;i<nos;++i)
		    if(i!=k && i!=0)
		      trp[j][k][i]=0.0;
		  mem=1;
		}

	    if(mem==0)
	      {
		for(i=0;i<nos;++i)
		  trp[j][k][i]=0.0;
		trp[j][k][k+1]=1.0;		
	      }
	  }
	ts=0.0;
	for(i=0;i<nos;++i)
	  ts+=trp[j][k][i];
	for(i=0;i<nos;++i)	  
	  trp[j][k][i]=trp[j][k][i]/ts;
	  
      }
  
    return; 
}
/*******************************************************************/
int trans(int sta)//transfers to true state number
{
  int ret=0;
  if(sta>M1)
    ret=(int)(sta-M1-1)/(M2+1)+1;

  return(ret);
}
/***********************************************************************/
 void memerror(char * fname)
  {
    printf("Out of memory in %s\n",fname);
    exit(8);
  }
 /**************************************************/
int testtrp(int nos, int noce, float ***trp)
{
	int i,j,k;
	float sum;
	for(k=0;k<noce-1;++k) 
		for(i=0;i<nos;++i)
		{
			sum=0.0;
			for(j=0;j<nos;++j)
				sum+=trp[k][i][j];
			if(sum<0.99 ||sum>1.001)
			{
				printf("error in testtrp %f\n",sum);
			exit(8);
			}
		}
		
		return;
}
/****************************************************************/
void rem_outlier(float *** signal, int ss, int nos, int noce, int nosig, float * mu, float *sig,int olp)
//Outliers are removed if the probability of obseving a value is <OLP for all possible states
{
	int outltest(float signal,float mu,float sig,int olp);
	int c=0;
int os,i,j,k,n;

if(Type==1 ||Type==5)
printf("For Signal type 1 and 5 outlier removal should not be called\n");
else
{
nos=trans(nos);

for(i=0;i<ss;++i)
for(j=0;j<noce;++j)
for(k=0;k<nosig;++k)
{
	os=0;
	for(n=0;n<nos;++n)
		os+=outltest(signal[i][j][k],mu[n],sig[n],olp);
	
	if(os==nos)
	{
		//printf("%f %d\n",signal[i][j][k],os);
		signal[i][j][k]=Md;
		++c;
	}

}
printf("Outlier critical value %d\t%d outliers removed\n",olp,c);
}
return;
}
/**********************************/
int outltest(float signal,float mu,float sig,int olp)
{
	int ret=0;
	float cv[10];
int fac=(int)(log10(Mixprop));
cv[0]=1.644853627;cv[1]=2.575829304;cv[2]=3.290526731;cv[3]=3.890591886;cv[4]=4.417173415;cv[5]=4.89163848;cv[6]=5.326723914;cv[7]=5.73072888;cv[8]=6.109410211;cv[9]=6.46695109;

	if(signal!=Md)
	{
		if(Type=4)	
		{
				if(fabs(signal-mu)/(Mixfac*sig)>cv[olp+fac])
				{
					ret=1;
					//printf(" %f %f %f =%f %f\n",signal,mu, Mixfac*sig,fabs(signal-mu)/(Mixfac*sig),cv[olp+fac]);
				}
		}
		else
	if(fabs(signal-mu)/sig>cv[olp])
		ret=1;
	}
	return(ret);
}
/*************************************************************************************************/
/*                        Reparametrization of data as EM                                        */
/*************************************************************************************************/

void repara_em(float *** signal, float *** probd, int noce, int ss, int nosig, int nos,int steps,float minf,float prior,float mins)
{
  void est_cs(float *mu,float *sig,float **cs,float ***signal,int mn, int ss, int nos, int nosig,float);
  void est_par(float *mu, float * sig, float ** cs, float *** signal, int mn, float mins, int ss, int nos, int nosig);
  void gen_rs(float **cs,int ss, int nos);
  void calc_probd_em(float minf,int ss,int nos,float *** probd,float ** cs, float * mu,float sig,int);

  int i,j,k;
  float mu[3],sig[3];
  float abp[3];
  float **cs; 
  float ec,ca=0.0;
  float min_mean;
 int rs;

  printf("Reparametrizing the intensities using a bayesian EM\nPrior %.2f minimum probabillity %.3f offsett %.3f\n",prior,minf,mins);
  cs=make2arr(ss,nos);
  for(i=0;i<noce;++i)
    {
      gen_rs(cs,ss,nos);
      sig[0]=0.0;
      for(j=0;j<steps;++j)
	{
	  est_par(mu,sig,cs,signal,i,0.0,ss,nos,nosig);

	  est_cs(mu,sig,cs,signal,i,ss,nos,nosig,prior);
	}
      
	min_mean=mins*sig[0];
	if(mu[1]>-min_mean)
	{
	//printf("resetting mu 1 from %.4f to -%.3f * %.3f= %.4f\n",mu[1],sig[0],mins,-min_mean);
++rs;
	mu[1]=-min_mean;
	}
	if(mu[2]<min_mean)
	{
++rs;
	//printf("resetting mu 2 from %.4f to %.3f * %.3f=%.4f\n",mu[2],sig[0],mins,min_mean);
	mu[2]=min_mean;
	}
      
      est_cs(mu,sig,cs,signal,i,ss,nos,nosig,prior);
     
      /*printf("Probe %d ",i);
      for(k=0;k<nos;++k)  
	printf("%.2f %.2f ",i,mu[k],sig[k]);
      printf("\n");*/
      calc_probd_em(minf,ss,nos,probd,cs,mu,sig[0],i);
      ec=0.0;
      for(j=0;j<ss;++j)
	{
	  for(k=0;k<nos;++k) 
	    { 
	      //printf("%f ",probd[j][i][k]);
	      if(k>0)ec+=probd[j][i][k];
	    }
	  //printf(" %.3f\n",signal[j][i][0]);
	}
      //printf("Probe %d calculated Expected carriers:%f\n",i,ec);
ca+=ec;
      //if(i>9) exit(8);
    }
  //exit(8);
printf("%d resets, mean number of carriers %.3f\n",rs,ca/noce);
  return;
}
/************************************************************/
void calc_probd_em(float minf,int ss,int nos,float *** probd,float ** cs, float * mu,float sig, int mn)
{
  int j,k;
  float thres=0.5;
  float diff;
  int cmp=0.0;
  for(k=1;k<nos;++k)
    if(fabs(mu[k])<thres*sig)
      {
	printf("Collapsing 0 and %d\n",k);
	for(j=0;j<ss;++j)
	  {
	    cs[j][0]+=cs[j][k];
	    cs[j][k]=0;
	  }
      }


  for(j=0;j<ss;++j)
    {
      for(k=0;k<nos;++k)
	if(cs[j][k]>=-0.05 && cs[j][k]<=1.05)
	  ;
	else
	  {
	    printf("Error 1 in calc_probd_em, cs[%d][%d]= %f\n",j,k,cs[j][k]);
	    exit(8);
	  }
      diff=0.0;cmp=0;
      for(k=0;k<nos;++k)
	if(cs[j][k]<minf)
	  {
	    diff+=(minf-cs[j][k]);
	    cs[j][k]=minf;
	  }
	else
	  if(cs[j][k]>2*minf)
	    ++cmp;
      for(k=0;k<nos;++k)
	{
	  if(cs[j][k]>2*minf)
	    if(cmp>0)
	      cs[j][k]-=diff/cmp;
	    else
	      {
		printf("Error 2 in calc_probd_em\n");
		exit(8);
	      }
	  probd[j][mn][k]=cs[j][k];
	  
	}
      
    }

  return;
}
/*******************************************************************/
void gen_rs(float **cs,int ss, int nos)
{
  int i;

  for(i=0;i<ss;++i)
    {
      switch (nos)
	{
	case 2:
	  cs[i][0]=RandDbl();
	  cs[i][1]=1-cs[i][0];
	  break;
	case 3:
	  cs[i][0]=RandDbl();
	  cs[i][1]=0.5*(1-cs[i][0]);
	  cs[i][2]=0.5*(1-cs[i][0]);
	  break;
	default:
	  printf("Not defined for number of states %d\n",nos);
	  exit(8);

	}
      //printf("%.2f %.2f %.2f\n",cs[i][0],cs[i][1],cs[i][2]);
    }
  return;
}
/*****************************************************************/
void est_par(float *mu, float * sig, float ** cs, float *** signal, int mn, float mins, int ss, int nos, int nosig)
{
  int i,j,k;
  float count[3];
  float gms,osd=sig[0];

  for(k=0;k<nos;++k)
    {mu[k]=0.0;sig[k]=0.0;count[k]=0.0;}

  for(i=0;i<ss;++i)
    for(j=0;j<nosig;++j)
      {
	for(k=1;k<nos;++k)
	  {
	    mu[k]+=cs[i][k]*signal[i][mn][j];
	    count[k]+=cs[i][k];
	  }
	sig[0]+=cs[i][0]*pow(signal[i][mn][j],2);
	count[0]+=cs[i][0];
      }

  for(k=1;k<nos;++k)
    if(count[k]>0)
      mu[k]=mu[k]/count[k];
  //printf("Test: %f %f\n",sig[0],count[0]);
  gms=sig[0];
  if(count[0]>0)
    sig[0]=sqrt(sig[0]/count[0]);
  
  //printf("test %f %f\n",mu[1],mu[0]);
  if(mu[1]>-osd*mins)
    {
      //printf("resetting mu 1 from %.4f to -%.3f * %.3f= %.4f\n",mu[1],osd,mins,-osd*mins);
      mu[1]=-osd*mins;
    }
  if(mu[2]<osd*mins)
    {
      //printf("resetting mu 2 from %.4f to %.3f * %.3f=%.4f\n",mu[2],osd,mins,osd*mins);
      mu[2]=osd*mins;

    }
  for(i=0;i<ss;++i)
    for(j=0;j<nosig;++j)
      for(k=1;k<nos;++k)
	sig[k]+=cs[i][k]*pow(signal[i][mn][j]-mu[k],2);

  for(k=1;k<nos;++k)
    {
      gms+=sig[k];
      if(count[k]>0)
	sig[k]=sqrt(sig[k]/count[k]);
    }

  gms=sqrt(gms/(ss*nosig));

  
  
  for(k=0;k<nos;++k)
    {/*
       if(sig[k]>2*gms)
       sig[k]=2*gms;
       if(sig[k]<gms/2)
       sig[k]=gms/2;
     */
      sig[k]=gms;
      
      //printf("state %d %.3f %.3f %.2f \n",k,mu[k],sig[k],count[k]);
    }
 
  return;
}
/**********************************************************************************************/
void est_cs(float *mu,float *sig,float **cs,float ***signal,int mn, int ss, int nos, int nosig,float pri)
{
  int i,j,k;
  float sum;

  double prob[3];
  double prior[3];

  prior[0]=1.0;
  prior[1]=pri;
  prior[2]=pri;

  for(i=0;i<ss;++i)
    {
      sum=0.0;
      for(k=0;k<nos;++k)
	{
	  prob[k]=prior[k];
	  for(j=0;j<nosig;++j)
	    if(signal[i][mn][j]!=Md)
	      prob[k]*=exp(-pow((signal[i][mn][j]-mu[k])/sig[k],2)/2)/(R2p*sig[k]);
	  if(prob[k]>0)
	    sum+=prob[k];
	  else
	    prob[k]=0;
	}
      if(sum>1e-10)
	for(k=0;k<nos;++k)
	  cs[i][k]=prob[k]/sum;
      else
	for(k=0;k<nos;++k)
	  cs[i][k]=1.0/nos;

      for(k=0;k<nos;++k)
      if(cs[i][k]>=-0.05 && cs[i][k]<=1.05)
	  ;
	else
	  {
	    printf("Error 1 in est_cs, cs= %f\n",cs[i][k]);
printf("Value=%f sum=%e\n",signal[i][mn][0],sum);
sum=0.0;
for(k=0;k<nos;++k)
{
printf("State %d prob %e prior %f mu %f sig %f\n",k,prob[k],prior[k],mu[k],sig[k]);
sum+=prob[k];
}
printf("Testsum %e\n",sum);
	    exit(8);
	  }
    }

  return;
}
